from typing import List
from localstack.cli import console
def print_table(column_headers,columns):
	C=columns;B=column_headers;from rich.table import Table;assert len(B)==len(C);A=Table(show_header=True,header_style='bold')
	for D in B:A.add_column(D)
	for E in zip(*(C)):A.add_row(*(E))
	console.print(A)


import logging,os,shutil,zipfile
from typing import Dict
from localstack.utils.files import new_tmp_dir,rm_rf
from localstack.utils.strings import short_uid
from localstack_ext.bootstrap.pods.constants import COMPRESSION_FORMAT
from localstack_ext.bootstrap.pods.utils.common import PodsConfigContext
LOG=logging.getLogger(__name__)
def extract_meta_and_state_archives(meta_archives,state_archives,config_context):
	G=False;F=config_context;E=meta_archives
	for C in [E,state_archives]:
		for (D,A) in C.items():
			if not A or not os.path.exists(A):raise Exception(f"Unable to find pods state/metamodel archive: {A}")
			with zipfile.ZipFile(A)as H:
				if C==E:B=F.get_version_meta_archive_path(version=D,with_format=G)
				else:B=F.get_version_state_archive_path(version=D,with_format=G)
				H.extractall(B);shutil.make_archive(base_name=B,format=COMPRESSION_FORMAT,root_dir=B);rm_rf(B);rm_rf(A);LOG.debug('Successfully extracted archive %s for version %s',C,D)
def merge_version_space(version_space_archive,config_context):
	C=version_space_archive;B=config_context;D=new_tmp_dir();G=short_uid();A=PodsConfigContext(pods_root_dir=D,pod_name=G)
	with zipfile.ZipFile(C)as H:H.extractall(A.get_pod_root_dir())
	shutil.copy(A.get_known_ver_path(),B.get_known_ver_path());shutil.copy(A.get_max_ver_path(),B.get_max_ver_path());I=A.get_versions_path();J=B.get_versions_path();K=A.get_obj_store_path();L=B.get_obj_store_path();M=(J,I),(L,K)
	for (N,E) in M:
		for F in os.listdir(E):O=os.path.join(E,F);P=os.path.join(N,F);shutil.copy(O,P)
	rm_rf(D);rm_rf(C)


import sys
from typing import IO, Iterable, Optional, Union

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

CustomAttributesHeader = str
EnableExplanationsHeader = str
EndpointName = str
Header = str
InferenceId = str
InputLocationHeader = str
InvocationTimeoutSecondsHeader = int
LogStreamArn = str
Message = str
RequestTTLSecondsHeader = int
StatusCode = int
TargetContainerHostnameHeader = str
TargetModelHeader = str
TargetVariantHeader = str


class InternalDependencyException(ServiceException):
    """Your request caused an exception with an internal dependency. Contact
    customer support.
    """

    code: str = "InternalDependencyException"
    sender_fault: bool = False
    status_code: int = 530


class InternalFailure(ServiceException):
    """An internal failure occurred."""

    code: str = "InternalFailure"
    sender_fault: bool = False
    status_code: int = 500


class ModelError(ServiceException):
    """Model (owned by the customer in the container) returned 4xx or 5xx error
    code.
    """

    code: str = "ModelError"
    sender_fault: bool = False
    status_code: int = 424
    OriginalStatusCode: Optional[StatusCode]
    OriginalMessage: Optional[Message]
    LogStreamArn: Optional[LogStreamArn]


class ModelNotReadyException(ServiceException):
    """Either a serverless endpoint variant's resources are still being
    provisioned, or a multi-model endpoint is still downloading or loading
    the target model. Wait and try your request again.
    """

    code: str = "ModelNotReadyException"
    sender_fault: bool = False
    status_code: int = 429


class ServiceUnavailable(ServiceException):
    """The service is unavailable. Try your call again."""

    code: str = "ServiceUnavailable"
    sender_fault: bool = False
    status_code: int = 503


class ValidationError(ServiceException):
    """Inspect your request and try again."""

    code: str = "ValidationError"
    sender_fault: bool = False
    status_code: int = 400


BodyBlob = bytes


class InvokeEndpointAsyncInput(ServiceRequest):
    EndpointName: EndpointName
    ContentType: Optional[Header]
    Accept: Optional[Header]
    CustomAttributes: Optional[CustomAttributesHeader]
    InferenceId: Optional[InferenceId]
    InputLocation: InputLocationHeader
    RequestTTLSeconds: Optional[RequestTTLSecondsHeader]
    InvocationTimeoutSeconds: Optional[InvocationTimeoutSecondsHeader]


class InvokeEndpointAsyncOutput(TypedDict, total=False):
    InferenceId: Optional[Header]
    OutputLocation: Optional[Header]


class InvokeEndpointInput(ServiceRequest):
    Body: IO[BodyBlob]
    EndpointName: EndpointName
    ContentType: Optional[Header]
    Accept: Optional[Header]
    CustomAttributes: Optional[CustomAttributesHeader]
    TargetModel: Optional[TargetModelHeader]
    TargetVariant: Optional[TargetVariantHeader]
    TargetContainerHostname: Optional[TargetContainerHostnameHeader]
    InferenceId: Optional[InferenceId]
    EnableExplanations: Optional[EnableExplanationsHeader]


class InvokeEndpointOutput(TypedDict, total=False):
    Body: Union[BodyBlob, IO[BodyBlob], Iterable[BodyBlob]]
    ContentType: Optional[Header]
    InvokedProductionVariant: Optional[Header]
    CustomAttributes: Optional[CustomAttributesHeader]


class SagemakerRuntimeApi:

    service = "sagemaker-runtime"
    version = "2017-05-13"

    @handler("InvokeEndpoint")
    def invoke_endpoint(
        self,
        context: RequestContext,
        endpoint_name: EndpointName,
        body: IO[BodyBlob],
        content_type: Header = None,
        accept: Header = None,
        custom_attributes: CustomAttributesHeader = None,
        target_model: TargetModelHeader = None,
        target_variant: TargetVariantHeader = None,
        target_container_hostname: TargetContainerHostnameHeader = None,
        inference_id: InferenceId = None,
        enable_explanations: EnableExplanationsHeader = None,
    ) -> InvokeEndpointOutput:
        """After you deploy a model into production using Amazon SageMaker hosting
        services, your client applications use this API to get inferences from
        the model hosted at the specified endpoint.

        For an overview of Amazon SageMaker, see `How It
        Works <https://docs.aws.amazon.com/sagemaker/latest/dg/how-it-works.html>`__.

        Amazon SageMaker strips all POST headers except those supported by the
        API. Amazon SageMaker might add additional headers. You should not rely
        on the behavior of headers outside those enumerated in the request
        syntax.

        Calls to ``InvokeEndpoint`` are authenticated by using Amazon Web
        Services Signature Version 4. For information, see `Authenticating
        Requests (Amazon Web Services Signature Version
        4) <https://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-authenticating-requests.html>`__
        in the *Amazon S3 API Reference*.

        A customer's model containers must respond to requests within 60
        seconds. The model itself can have a maximum processing time of 60
        seconds before responding to invocations. If your model is going to take
        50-60 seconds of processing time, the SDK socket timeout should be set
        to be 70 seconds.

        Endpoints are scoped to an individual account, and are not public. The
        URL does not contain the account ID, but Amazon SageMaker determines the
        account ID from the authentication token that is supplied by the caller.

        :param endpoint_name: The name of the endpoint that you specified when you created the
        endpoint using the
        `CreateEndpoint <https://docs.
        :param body: Provides input data, in the format specified in the ``ContentType``
        request header.
        :param content_type: The MIME type of the input data in the request body.
        :param accept: The desired MIME type of the inference in the response.
        :param custom_attributes: Provides additional information about a request for an inference
        submitted to a model hosted at an Amazon SageMaker endpoint.
        :param target_model: The model to request for inference when invoking a multi-model endpoint.
        :param target_variant: Specify the production variant to send the inference request to when
        invoking an endpoint that is running two or more variants.
        :param target_container_hostname: If the endpoint hosts multiple containers and is configured to use
        direct invocation, this parameter specifies the host name of the
        container to invoke.
        :param inference_id: If you provide a value, it is added to the captured data when you enable
        data capture on the endpoint.
        :param enable_explanations: An optional JMESPath expression used to override the
        ``EnableExplanations`` parameter of the ``ClarifyExplainerConfig`` API.
        :returns: InvokeEndpointOutput
        :raises InternalFailure:
        :raises ServiceUnavailable:
        :raises ValidationError:
        :raises ModelError:
        :raises InternalDependencyException:
        :raises ModelNotReadyException:
        """
        raise NotImplementedError

    @handler("InvokeEndpointAsync")
    def invoke_endpoint_async(
        self,
        context: RequestContext,
        endpoint_name: EndpointName,
        input_location: InputLocationHeader,
        content_type: Header = None,
        accept: Header = None,
        custom_attributes: CustomAttributesHeader = None,
        inference_id: InferenceId = None,
        request_ttl_seconds: RequestTTLSecondsHeader = None,
        invocation_timeout_seconds: InvocationTimeoutSecondsHeader = None,
    ) -> InvokeEndpointAsyncOutput:
        """After you deploy a model into production using Amazon SageMaker hosting
        services, your client applications use this API to get inferences from
        the model hosted at the specified endpoint in an asynchronous manner.

        Inference requests sent to this API are enqueued for asynchronous
        processing. The processing of the inference request may or may not
        complete before you receive a response from this API. The response from
        this API will not contain the result of the inference request but
        contain information about where you can locate it.

        Amazon SageMaker strips all ``POST`` headers except those supported by
        the API. Amazon SageMaker might add additional headers. You should not
        rely on the behavior of headers outside those enumerated in the request
        syntax.

        Calls to ``InvokeEndpointAsync`` are authenticated by using Amazon Web
        Services Signature Version 4. For information, see `Authenticating
        Requests (Amazon Web Services Signature Version
        4) <https://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-authenticating-requests.html>`__
        in the *Amazon S3 API Reference*.

        :param endpoint_name: The name of the endpoint that you specified when you created the
        endpoint using the
        ```CreateEndpoint`` <https://docs.
        :param input_location: The Amazon S3 URI where the inference request payload is stored.
        :param content_type: The MIME type of the input data in the request body.
        :param accept: The desired MIME type of the inference in the response.
        :param custom_attributes: Provides additional information about a request for an inference
        submitted to a model hosted at an Amazon SageMaker endpoint.
        :param inference_id: The identifier for the inference request.
        :param request_ttl_seconds: Maximum age in seconds a request can be in the queue before it is marked
        as expired.
        :param invocation_timeout_seconds: Maximum amount of time in seconds a request can be processed before it
        is marked as expired.
        :returns: InvokeEndpointAsyncOutput
        :raises InternalFailure:
        :raises ServiceUnavailable:
        :raises ValidationError:
        """
        raise NotImplementedError



import sys
from datetime import datetime
from typing import Dict, List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

ApplicationDescription = str
ApplicationMaintenanceWindowEndTime = str
ApplicationMaintenanceWindowStartTime = str
ApplicationName = str
AuthorizedUrl = str
BasePath = str
BooleanObject = bool
BucketARN = str
CodeMD5 = str
ConditionalToken = str
DatabaseARN = str
ErrorMessage = str
FileKey = str
Id = str
InAppStreamName = str
InAppTableName = str
InputParallelismCount = int
JobPlanDescription = str
KinesisAnalyticsARN = str
ListApplicationVersionsInputLimit = int
ListApplicationsInputLimit = int
ListSnapshotsInputLimit = int
LogStreamARN = str
MavenArtifactId = str
MavenGroupId = str
MavenVersion = str
NextToken = str
ObjectVersion = str
Parallelism = int
ParallelismPerKPU = int
ParsedInputRecordField = str
ProcessedInputRecord = str
PropertyKey = str
PropertyValue = str
RawInputRecord = str
RecordColumnDelimiter = str
RecordColumnMapping = str
RecordColumnName = str
RecordColumnSqlType = str
RecordEncoding = str
RecordRowDelimiter = str
RecordRowPath = str
ResourceARN = str
RoleARN = str
SecurityGroupId = str
SnapshotName = str
SubnetId = str
TagKey = str
TagValue = str
TextContent = str
VpcId = str


class ApplicationMode(str):
    STREAMING = "STREAMING"
    INTERACTIVE = "INTERACTIVE"


class ApplicationRestoreType(str):
    SKIP_RESTORE_FROM_SNAPSHOT = "SKIP_RESTORE_FROM_SNAPSHOT"
    RESTORE_FROM_LATEST_SNAPSHOT = "RESTORE_FROM_LATEST_SNAPSHOT"
    RESTORE_FROM_CUSTOM_SNAPSHOT = "RESTORE_FROM_CUSTOM_SNAPSHOT"


class ApplicationStatus(str):
    DELETING = "DELETING"
    STARTING = "STARTING"
    STOPPING = "STOPPING"
    READY = "READY"
    RUNNING = "RUNNING"
    UPDATING = "UPDATING"
    AUTOSCALING = "AUTOSCALING"
    FORCE_STOPPING = "FORCE_STOPPING"
    ROLLING_BACK = "ROLLING_BACK"
    MAINTENANCE = "MAINTENANCE"
    ROLLED_BACK = "ROLLED_BACK"


class ArtifactType(str):
    UDF = "UDF"
    DEPENDENCY_JAR = "DEPENDENCY_JAR"


class CodeContentType(str):
    PLAINTEXT = "PLAINTEXT"
    ZIPFILE = "ZIPFILE"


class ConfigurationType(str):
    DEFAULT = "DEFAULT"
    CUSTOM = "CUSTOM"


class InputStartingPosition(str):
    NOW = "NOW"
    TRIM_HORIZON = "TRIM_HORIZON"
    LAST_STOPPED_POINT = "LAST_STOPPED_POINT"


class LogLevel(str):
    INFO = "INFO"
    WARN = "WARN"
    ERROR = "ERROR"
    DEBUG = "DEBUG"


class MetricsLevel(str):
    APPLICATION = "APPLICATION"
    TASK = "TASK"
    OPERATOR = "OPERATOR"
    PARALLELISM = "PARALLELISM"


class RecordFormatType(str):
    JSON = "JSON"
    CSV = "CSV"


class RuntimeEnvironment(str):
    SQL_1_0 = "SQL-1_0"
    FLINK_1_6 = "FLINK-1_6"
    FLINK_1_8 = "FLINK-1_8"
    ZEPPELIN_FLINK_1_0 = "ZEPPELIN-FLINK-1_0"
    FLINK_1_11 = "FLINK-1_11"
    FLINK_1_13 = "FLINK-1_13"
    ZEPPELIN_FLINK_2_0 = "ZEPPELIN-FLINK-2_0"
    FLINK_1_15 = "FLINK-1_15"


class SnapshotStatus(str):
    CREATING = "CREATING"
    READY = "READY"
    DELETING = "DELETING"
    FAILED = "FAILED"


class UrlType(str):
    FLINK_DASHBOARD_URL = "FLINK_DASHBOARD_URL"
    ZEPPELIN_UI_URL = "ZEPPELIN_UI_URL"


class CodeValidationException(ServiceException):
    """The user-provided application code (query) is not valid. This can be a
    simple syntax error.
    """

    code: str = "CodeValidationException"
    sender_fault: bool = False
    status_code: int = 400


class ConcurrentModificationException(ServiceException):
    """Exception thrown as a result of concurrent modifications to an
    application. This error can be the result of attempting to modify an
    application without using the current application ID.
    """

    code: str = "ConcurrentModificationException"
    sender_fault: bool = False
    status_code: int = 400


class InvalidApplicationConfigurationException(ServiceException):
    """The user-provided application configuration is not valid."""

    code: str = "InvalidApplicationConfigurationException"
    sender_fault: bool = False
    status_code: int = 400


class InvalidArgumentException(ServiceException):
    """The specified input parameter value is not valid."""

    code: str = "InvalidArgumentException"
    sender_fault: bool = False
    status_code: int = 400


class InvalidRequestException(ServiceException):
    """The request JSON is not valid for the operation."""

    code: str = "InvalidRequestException"
    sender_fault: bool = False
    status_code: int = 400


class LimitExceededException(ServiceException):
    """The number of allowed resources has been exceeded."""

    code: str = "LimitExceededException"
    sender_fault: bool = False
    status_code: int = 400


class ResourceInUseException(ServiceException):
    """The application is not available for this operation."""

    code: str = "ResourceInUseException"
    sender_fault: bool = False
    status_code: int = 400


class ResourceNotFoundException(ServiceException):
    """Specified application can't be found."""

    code: str = "ResourceNotFoundException"
    sender_fault: bool = False
    status_code: int = 400


class ResourceProvisionedThroughputExceededException(ServiceException):
    """Discovery failed to get a record from the streaming source because of
    the Kinesis Streams ``ProvisionedThroughputExceededException``. For more
    information, see
    `GetRecords <http://docs.aws.amazon.com/kinesis/latest/APIReference/API_GetRecords.html>`__
    in the Amazon Kinesis Streams API Reference.
    """

    code: str = "ResourceProvisionedThroughputExceededException"
    sender_fault: bool = False
    status_code: int = 400


class ServiceUnavailableException(ServiceException):
    """The service cannot complete the request."""

    code: str = "ServiceUnavailableException"
    sender_fault: bool = False
    status_code: int = 400


class TooManyTagsException(ServiceException):
    """Application created with too many tags, or too many tags added to an
    application. Note that the maximum number of application tags includes
    system tags. The maximum number of user-defined application tags is 50.
    """

    code: str = "TooManyTagsException"
    sender_fault: bool = False
    status_code: int = 400


ProcessedInputRecords = List[ProcessedInputRecord]
RawInputRecords = List[RawInputRecord]


class UnableToDetectSchemaException(ServiceException):
    """The data format is not valid. Kinesis Data Analytics cannot detect the
    schema for the given streaming source.
    """

    code: str = "UnableToDetectSchemaException"
    sender_fault: bool = False
    status_code: int = 400
    RawInputRecords: Optional[RawInputRecords]
    ProcessedInputRecords: Optional[ProcessedInputRecords]


class UnsupportedOperationException(ServiceException):
    """The request was rejected because a specified parameter is not supported
    or a specified resource is not valid for this operation.
    """

    code: str = "UnsupportedOperationException"
    sender_fault: bool = False
    status_code: int = 400


class CloudWatchLoggingOption(TypedDict, total=False):
    """Provides a description of Amazon CloudWatch logging options, including
    the log stream Amazon Resource Name (ARN).
    """

    LogStreamARN: LogStreamARN


ApplicationVersionId = int


class AddApplicationCloudWatchLoggingOptionRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: Optional[ApplicationVersionId]
    CloudWatchLoggingOption: CloudWatchLoggingOption
    ConditionalToken: Optional[ConditionalToken]


class CloudWatchLoggingOptionDescription(TypedDict, total=False):
    """Describes the Amazon CloudWatch logging option."""

    CloudWatchLoggingOptionId: Optional[Id]
    LogStreamARN: LogStreamARN
    RoleARN: Optional[RoleARN]


CloudWatchLoggingOptionDescriptions = List[CloudWatchLoggingOptionDescription]


class AddApplicationCloudWatchLoggingOptionResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]
    CloudWatchLoggingOptionDescriptions: Optional[CloudWatchLoggingOptionDescriptions]


class InputLambdaProcessor(TypedDict, total=False):
    """An object that contains the Amazon Resource Name (ARN) of the Amazon
    Lambda function that is used to preprocess records in the stream in a
    SQL-based Kinesis Data Analytics application.
    """

    ResourceARN: ResourceARN


class InputProcessingConfiguration(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes a
    processor that is used to preprocess the records in the stream before
    being processed by your application code. Currently, the only input
    processor available is `Amazon
    Lambda <https://docs.aws.amazon.com/lambda/>`__.
    """

    InputLambdaProcessor: InputLambdaProcessor


class AddApplicationInputProcessingConfigurationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    InputId: Id
    InputProcessingConfiguration: InputProcessingConfiguration


class InputLambdaProcessorDescription(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, an object that
    contains the Amazon Resource Name (ARN) of the Amazon Lambda function
    that is used to preprocess records in the stream.
    """

    ResourceARN: ResourceARN
    RoleARN: Optional[RoleARN]


class InputProcessingConfigurationDescription(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, provides the
    configuration information about an input processor. Currently, the only
    input processor available is `Amazon
    Lambda <https://docs.aws.amazon.com/lambda/>`__.
    """

    InputLambdaProcessorDescription: Optional[InputLambdaProcessorDescription]


class AddApplicationInputProcessingConfigurationResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]
    InputId: Optional[Id]
    InputProcessingConfigurationDescription: Optional[InputProcessingConfigurationDescription]


class RecordColumn(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes the
    mapping of each data element in the streaming source to the
    corresponding column in the in-application stream.

    Also used to describe the format of the reference data source.
    """

    Name: RecordColumnName
    Mapping: Optional[RecordColumnMapping]
    SqlType: RecordColumnSqlType


RecordColumns = List[RecordColumn]


class CSVMappingParameters(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, provides additional
    mapping information when the record format uses delimiters, such as CSV.
    For example, the following sample records use CSV format, where the
    records use the *'\n'* as the row delimiter and a comma (",") as the
    column delimiter:

    ``"name1", "address1"``

    ``"name2", "address2"``
    """

    RecordRowDelimiter: RecordRowDelimiter
    RecordColumnDelimiter: RecordColumnDelimiter


class JSONMappingParameters(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, provides additional
    mapping information when JSON is the record format on the streaming
    source.
    """

    RecordRowPath: RecordRowPath


class MappingParameters(TypedDict, total=False):
    """When you configure a SQL-based Kinesis Data Analytics application's
    input at the time of creating or updating an application, provides
    additional mapping information specific to the record format (such as
    JSON, CSV, or record fields delimited by some delimiter) on the
    streaming source.
    """

    JSONMappingParameters: Optional[JSONMappingParameters]
    CSVMappingParameters: Optional[CSVMappingParameters]


class RecordFormat(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes the record
    format and relevant mapping information that should be applied to
    schematize the records on the stream.
    """

    RecordFormatType: RecordFormatType
    MappingParameters: Optional[MappingParameters]


class SourceSchema(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes the format
    of the data in the streaming source, and how each data element maps to
    corresponding columns created in the in-application stream.
    """

    RecordFormat: RecordFormat
    RecordEncoding: Optional[RecordEncoding]
    RecordColumns: RecordColumns


class InputParallelism(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes the number
    of in-application streams to create for a given streaming source.
    """

    Count: Optional[InputParallelismCount]


class KinesisFirehoseInput(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, identifies a Kinesis
    Data Firehose delivery stream as the streaming source. You provide the
    delivery stream's Amazon Resource Name (ARN).
    """

    ResourceARN: ResourceARN


class KinesisStreamsInput(TypedDict, total=False):
    """Identifies a Kinesis data stream as the streaming source. You provide
    the stream's Amazon Resource Name (ARN).
    """

    ResourceARN: ResourceARN


class Input(TypedDict, total=False):
    """When you configure the application input for a SQL-based Kinesis Data
    Analytics application, you specify the streaming source, the
    in-application stream name that is created, and the mapping between the
    two.
    """

    NamePrefix: InAppStreamName
    InputProcessingConfiguration: Optional[InputProcessingConfiguration]
    KinesisStreamsInput: Optional[KinesisStreamsInput]
    KinesisFirehoseInput: Optional[KinesisFirehoseInput]
    InputParallelism: Optional[InputParallelism]
    InputSchema: SourceSchema


class AddApplicationInputRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    Input: Input


class InputStartingPositionConfiguration(TypedDict, total=False):
    """Describes the point at which the application reads from the streaming
    source.
    """

    InputStartingPosition: Optional[InputStartingPosition]


class KinesisFirehoseInputDescription(TypedDict, total=False):
    """Describes the Amazon Kinesis Data Firehose delivery stream that is
    configured as the streaming source in the application input
    configuration.
    """

    ResourceARN: ResourceARN
    RoleARN: Optional[RoleARN]


class KinesisStreamsInputDescription(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes the
    Kinesis data stream that is configured as the streaming source in the
    application input configuration.
    """

    ResourceARN: ResourceARN
    RoleARN: Optional[RoleARN]


InAppStreamNames = List[InAppStreamName]


class InputDescription(TypedDict, total=False):
    """Describes the application input configuration for a SQL-based Kinesis
    Data Analytics application.
    """

    InputId: Optional[Id]
    NamePrefix: Optional[InAppStreamName]
    InAppStreamNames: Optional[InAppStreamNames]
    InputProcessingConfigurationDescription: Optional[InputProcessingConfigurationDescription]
    KinesisStreamsInputDescription: Optional[KinesisStreamsInputDescription]
    KinesisFirehoseInputDescription: Optional[KinesisFirehoseInputDescription]
    InputSchema: Optional[SourceSchema]
    InputParallelism: Optional[InputParallelism]
    InputStartingPositionConfiguration: Optional[InputStartingPositionConfiguration]


InputDescriptions = List[InputDescription]


class AddApplicationInputResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]
    InputDescriptions: Optional[InputDescriptions]


class DestinationSchema(TypedDict, total=False):
    """Describes the data format when records are written to the destination in
    a SQL-based Kinesis Data Analytics application.
    """

    RecordFormatType: RecordFormatType


class LambdaOutput(TypedDict, total=False):
    """When you configure a SQL-based Kinesis Data Analytics application's
    output, identifies an Amazon Lambda function as the destination. You
    provide the function Amazon Resource Name (ARN) of the Lambda function.
    """

    ResourceARN: ResourceARN


class KinesisFirehoseOutput(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, when configuring
    application output, identifies a Kinesis Data Firehose delivery stream
    as the destination. You provide the stream Amazon Resource Name (ARN) of
    the delivery stream.
    """

    ResourceARN: ResourceARN


class KinesisStreamsOutput(TypedDict, total=False):
    """When you configure a SQL-based Kinesis Data Analytics application's
    output, identifies a Kinesis data stream as the destination. You provide
    the stream Amazon Resource Name (ARN).
    """

    ResourceARN: ResourceARN


class Output(TypedDict, total=False):
    """Describes a SQL-based Kinesis Data Analytics application's output
    configuration, in which you identify an in-application stream and a
    destination where you want the in-application stream data to be written.
    The destination can be a Kinesis data stream or a Kinesis Data Firehose
    delivery stream.
    """

    Name: InAppStreamName
    KinesisStreamsOutput: Optional[KinesisStreamsOutput]
    KinesisFirehoseOutput: Optional[KinesisFirehoseOutput]
    LambdaOutput: Optional[LambdaOutput]
    DestinationSchema: DestinationSchema


class AddApplicationOutputRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    Output: Output


class LambdaOutputDescription(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application's output, describes
    the Amazon Lambda function that is configured as its destination.
    """

    ResourceARN: ResourceARN
    RoleARN: Optional[RoleARN]


class KinesisFirehoseOutputDescription(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application's output, describes
    the Kinesis Data Firehose delivery stream that is configured as its
    destination.
    """

    ResourceARN: ResourceARN
    RoleARN: Optional[RoleARN]


class KinesisStreamsOutputDescription(TypedDict, total=False):
    """For an SQL-based Kinesis Data Analytics application's output, describes
    the Kinesis data stream that is configured as its destination.
    """

    ResourceARN: ResourceARN
    RoleARN: Optional[RoleARN]


class OutputDescription(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes the
    application output configuration, which includes the in-application
    stream name and the destination where the stream data is written. The
    destination can be a Kinesis data stream or a Kinesis Data Firehose
    delivery stream.
    """

    OutputId: Optional[Id]
    Name: Optional[InAppStreamName]
    KinesisStreamsOutputDescription: Optional[KinesisStreamsOutputDescription]
    KinesisFirehoseOutputDescription: Optional[KinesisFirehoseOutputDescription]
    LambdaOutputDescription: Optional[LambdaOutputDescription]
    DestinationSchema: Optional[DestinationSchema]


OutputDescriptions = List[OutputDescription]


class AddApplicationOutputResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]
    OutputDescriptions: Optional[OutputDescriptions]


class S3ReferenceDataSource(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, identifies the
    Amazon S3 bucket and object that contains the reference data.

    A Kinesis Data Analytics application loads reference data only once. If
    the data changes, you call the UpdateApplication operation to trigger
    reloading of data into your application.
    """

    BucketARN: Optional[BucketARN]
    FileKey: Optional[FileKey]


class ReferenceDataSource(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes the
    reference data source by providing the source information (Amazon S3
    bucket name and object key name), the resulting in-application table
    name that is created, and the necessary schema to map the data elements
    in the Amazon S3 object to the in-application table.
    """

    TableName: InAppTableName
    S3ReferenceDataSource: Optional[S3ReferenceDataSource]
    ReferenceSchema: SourceSchema


class AddApplicationReferenceDataSourceRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    ReferenceDataSource: ReferenceDataSource


class S3ReferenceDataSourceDescription(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, provides the bucket
    name and object key name that stores the reference data.
    """

    BucketARN: BucketARN
    FileKey: FileKey
    ReferenceRoleARN: Optional[RoleARN]


class ReferenceDataSourceDescription(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes the
    reference data source configured for an application.
    """

    ReferenceId: Id
    TableName: InAppTableName
    S3ReferenceDataSourceDescription: S3ReferenceDataSourceDescription
    ReferenceSchema: Optional[SourceSchema]


ReferenceDataSourceDescriptions = List[ReferenceDataSourceDescription]


class AddApplicationReferenceDataSourceResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]
    ReferenceDataSourceDescriptions: Optional[ReferenceDataSourceDescriptions]


SecurityGroupIds = List[SecurityGroupId]
SubnetIds = List[SubnetId]


class VpcConfiguration(TypedDict, total=False):
    """Describes the parameters of a VPC used by the application."""

    SubnetIds: SubnetIds
    SecurityGroupIds: SecurityGroupIds


class AddApplicationVpcConfigurationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: Optional[ApplicationVersionId]
    VpcConfiguration: VpcConfiguration
    ConditionalToken: Optional[ConditionalToken]


class VpcConfigurationDescription(TypedDict, total=False):
    """Describes the parameters of a VPC used by the application."""

    VpcConfigurationId: Id
    VpcId: VpcId
    SubnetIds: SubnetIds
    SecurityGroupIds: SecurityGroupIds


class AddApplicationVpcConfigurationResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]
    VpcConfigurationDescription: Optional[VpcConfigurationDescription]


class S3ContentLocation(TypedDict, total=False):
    """For a Kinesis Data Analytics application provides a description of an
    Amazon S3 object, including the Amazon Resource Name (ARN) of the S3
    bucket, the name of the Amazon S3 object that contains the data, and the
    version number of the Amazon S3 object that contains the data.
    """

    BucketARN: BucketARN
    FileKey: FileKey
    ObjectVersion: Optional[ObjectVersion]


ZipFileContent = bytes


class CodeContent(TypedDict, total=False):
    """Specifies either the application code, or the location of the
    application code, for a Flink-based Kinesis Data Analytics application.
    """

    TextContent: Optional[TextContent]
    ZipFileContent: Optional[ZipFileContent]
    S3ContentLocation: Optional[S3ContentLocation]


class ApplicationCodeConfiguration(TypedDict, total=False):
    """Describes code configuration for an application."""

    CodeContent: Optional[CodeContent]
    CodeContentType: CodeContentType


class S3ApplicationCodeLocationDescription(TypedDict, total=False):
    """Describes the location of an application's code stored in an S3 bucket."""

    BucketARN: BucketARN
    FileKey: FileKey
    ObjectVersion: Optional[ObjectVersion]


CodeSize = int


class CodeContentDescription(TypedDict, total=False):
    """Describes details about the code of a Kinesis Data Analytics
    application.
    """

    TextContent: Optional[TextContent]
    CodeMD5: Optional[CodeMD5]
    CodeSize: Optional[CodeSize]
    S3ApplicationCodeLocationDescription: Optional[S3ApplicationCodeLocationDescription]


class ApplicationCodeConfigurationDescription(TypedDict, total=False):
    """Describes code configuration for an application."""

    CodeContentType: CodeContentType
    CodeContentDescription: Optional[CodeContentDescription]


class S3ContentLocationUpdate(TypedDict, total=False):
    """Describes an update for the Amazon S3 code content location for an
    application.
    """

    BucketARNUpdate: Optional[BucketARN]
    FileKeyUpdate: Optional[FileKey]
    ObjectVersionUpdate: Optional[ObjectVersion]


class CodeContentUpdate(TypedDict, total=False):
    """Describes an update to the code of an application. Not supported for
    Apache Zeppelin.
    """

    TextContentUpdate: Optional[TextContent]
    ZipFileContentUpdate: Optional[ZipFileContent]
    S3ContentLocationUpdate: Optional[S3ContentLocationUpdate]


class ApplicationCodeConfigurationUpdate(TypedDict, total=False):
    """Describes code configuration updates for an application. This is
    supported for a Flink-based Kinesis Data Analytics application or a
    SQL-based Kinesis Data Analytics application.
    """

    CodeContentTypeUpdate: Optional[CodeContentType]
    CodeContentUpdate: Optional[CodeContentUpdate]


class MavenReference(TypedDict, total=False):
    """The information required to specify a Maven reference. You can use Maven
    references to specify dependency JAR files.
    """

    GroupId: MavenGroupId
    ArtifactId: MavenArtifactId
    Version: MavenVersion


class CustomArtifactConfiguration(TypedDict, total=False):
    """Specifies dependency JARs, as well as JAR files that contain
    user-defined functions (UDF).
    """

    ArtifactType: ArtifactType
    S3ContentLocation: Optional[S3ContentLocation]
    MavenReference: Optional[MavenReference]


CustomArtifactsConfigurationList = List[CustomArtifactConfiguration]


class S3ContentBaseLocation(TypedDict, total=False):
    """The S3 bucket that holds the application information."""

    BucketARN: BucketARN
    BasePath: Optional[BasePath]


class DeployAsApplicationConfiguration(TypedDict, total=False):
    """The information required to deploy a Kinesis Data Analytics Studio
    notebook as an application with durable state.
    """

    S3ContentLocation: S3ContentBaseLocation


class GlueDataCatalogConfiguration(TypedDict, total=False):
    """The configuration of the Glue Data Catalog that you use for Apache Flink
    SQL queries and table API transforms that you write in an application.
    """

    DatabaseARN: DatabaseARN


class CatalogConfiguration(TypedDict, total=False):
    """The configuration parameters for the default Amazon Glue database. You
    use this database for SQL queries that you write in a Kinesis Data
    Analytics Studio notebook.
    """

    GlueDataCatalogConfiguration: GlueDataCatalogConfiguration


class ZeppelinMonitoringConfiguration(TypedDict, total=False):
    """Describes configuration parameters for Amazon CloudWatch logging for a
    Kinesis Data Analytics Studio notebook. For more information about
    CloudWatch logging, see
    `Monitoring <https://docs.aws.amazon.com/kinesisanalytics/latest/java/monitoring-overview.html>`__.
    """

    LogLevel: LogLevel


class ZeppelinApplicationConfiguration(TypedDict, total=False):
    """The configuration of a Kinesis Data Analytics Studio notebook."""

    MonitoringConfiguration: Optional[ZeppelinMonitoringConfiguration]
    CatalogConfiguration: Optional[CatalogConfiguration]
    DeployAsApplicationConfiguration: Optional[DeployAsApplicationConfiguration]
    CustomArtifactsConfiguration: Optional[CustomArtifactsConfigurationList]


VpcConfigurations = List[VpcConfiguration]


class ApplicationSnapshotConfiguration(TypedDict, total=False):
    """Describes whether snapshots are enabled for a Flink-based Kinesis Data
    Analytics application.
    """

    SnapshotsEnabled: BooleanObject


PropertyMap = Dict[PropertyKey, PropertyValue]


class PropertyGroup(TypedDict, total=False):
    """Property key-value pairs passed into an application."""

    PropertyGroupId: Id
    PropertyMap: PropertyMap


PropertyGroups = List[PropertyGroup]


class EnvironmentProperties(TypedDict, total=False):
    """Describes execution properties for a Flink-based Kinesis Data Analytics
    application.
    """

    PropertyGroups: PropertyGroups


class ParallelismConfiguration(TypedDict, total=False):
    """Describes parameters for how a Flink-based Kinesis Data Analytics
    application executes multiple tasks simultaneously. For more information
    about parallelism, see `Parallel
    Execution <https://ci.apache.org/projects/flink/flink-docs-release-1.8/dev/parallel.html>`__
    in the `Apache Flink
    Documentation <https://ci.apache.org/projects/flink/flink-docs-release-1.8/>`__.
    """

    ConfigurationType: ConfigurationType
    Parallelism: Optional[Parallelism]
    ParallelismPerKPU: Optional[ParallelismPerKPU]
    AutoScalingEnabled: Optional[BooleanObject]


class MonitoringConfiguration(TypedDict, total=False):
    """Describes configuration parameters for Amazon CloudWatch logging for an
    application. For more information about CloudWatch logging, see
    `Monitoring <https://docs.aws.amazon.com/kinesisanalytics/latest/java/monitoring-overview.html>`__.
    """

    ConfigurationType: ConfigurationType
    MetricsLevel: Optional[MetricsLevel]
    LogLevel: Optional[LogLevel]


MinPauseBetweenCheckpoints = int
CheckpointInterval = int


class CheckpointConfiguration(TypedDict, total=False):
    """Describes an application's checkpointing configuration. Checkpointing is
    the process of persisting application state for fault tolerance. For
    more information, see `Checkpoints for Fault
    Tolerance <https://ci.apache.org/projects/flink/flink-docs-release-1.8/concepts/programming-model.html#checkpoints-for-fault-tolerance>`__
    in the `Apache Flink
    Documentation <https://ci.apache.org/projects/flink/flink-docs-release-1.8/>`__.
    """

    ConfigurationType: ConfigurationType
    CheckpointingEnabled: Optional[BooleanObject]
    CheckpointInterval: Optional[CheckpointInterval]
    MinPauseBetweenCheckpoints: Optional[MinPauseBetweenCheckpoints]


class FlinkApplicationConfiguration(TypedDict, total=False):
    """Describes configuration parameters for a Flink-based Kinesis Data
    Analytics application or a Studio notebook.
    """

    CheckpointConfiguration: Optional[CheckpointConfiguration]
    MonitoringConfiguration: Optional[MonitoringConfiguration]
    ParallelismConfiguration: Optional[ParallelismConfiguration]


ReferenceDataSources = List[ReferenceDataSource]
Outputs = List[Output]
Inputs = List[Input]


class SqlApplicationConfiguration(TypedDict, total=False):
    """Describes the inputs, outputs, and reference data sources for a
    SQL-based Kinesis Data Analytics application.
    """

    Inputs: Optional[Inputs]
    Outputs: Optional[Outputs]
    ReferenceDataSources: Optional[ReferenceDataSources]


class ApplicationConfiguration(TypedDict, total=False):
    """Specifies the creation parameters for a Kinesis Data Analytics
    application.
    """

    SqlApplicationConfiguration: Optional[SqlApplicationConfiguration]
    FlinkApplicationConfiguration: Optional[FlinkApplicationConfiguration]
    EnvironmentProperties: Optional[EnvironmentProperties]
    ApplicationCodeConfiguration: Optional[ApplicationCodeConfiguration]
    ApplicationSnapshotConfiguration: Optional[ApplicationSnapshotConfiguration]
    VpcConfigurations: Optional[VpcConfigurations]
    ZeppelinApplicationConfiguration: Optional[ZeppelinApplicationConfiguration]


class CustomArtifactConfigurationDescription(TypedDict, total=False):
    """Specifies a dependency JAR or a JAR of user-defined functions."""

    ArtifactType: Optional[ArtifactType]
    S3ContentLocationDescription: Optional[S3ContentLocation]
    MavenReferenceDescription: Optional[MavenReference]


CustomArtifactsConfigurationDescriptionList = List[CustomArtifactConfigurationDescription]


class S3ContentBaseLocationDescription(TypedDict, total=False):
    """The description of the S3 base location that holds the application."""

    BucketARN: BucketARN
    BasePath: Optional[BasePath]


class DeployAsApplicationConfigurationDescription(TypedDict, total=False):
    """The configuration information required to deploy an Amazon Data
    Analytics Studio notebook as an application with durable state.
    """

    S3ContentLocationDescription: S3ContentBaseLocationDescription


class GlueDataCatalogConfigurationDescription(TypedDict, total=False):
    """The configuration of the Glue Data Catalog that you use for Apache Flink
    SQL queries and table API transforms that you write in an application.
    """

    DatabaseARN: DatabaseARN


class CatalogConfigurationDescription(TypedDict, total=False):
    """The configuration parameters for the default Amazon Glue database. You
    use this database for Apache Flink SQL queries and table API transforms
    that you write in a Kinesis Data Analytics Studio notebook.
    """

    GlueDataCatalogConfigurationDescription: GlueDataCatalogConfigurationDescription


class ZeppelinMonitoringConfigurationDescription(TypedDict, total=False):
    """The monitoring configuration for Apache Zeppelin within a Kinesis Data
    Analytics Studio notebook.
    """

    LogLevel: Optional[LogLevel]


class ZeppelinApplicationConfigurationDescription(TypedDict, total=False):
    """The configuration of a Kinesis Data Analytics Studio notebook."""

    MonitoringConfigurationDescription: ZeppelinMonitoringConfigurationDescription
    CatalogConfigurationDescription: Optional[CatalogConfigurationDescription]
    DeployAsApplicationConfigurationDescription: Optional[
        DeployAsApplicationConfigurationDescription
    ]
    CustomArtifactsConfigurationDescription: Optional[CustomArtifactsConfigurationDescriptionList]


VpcConfigurationDescriptions = List[VpcConfigurationDescription]


class ApplicationSnapshotConfigurationDescription(TypedDict, total=False):
    """Describes whether snapshots are enabled for a Flink-based Kinesis Data
    Analytics application.
    """

    SnapshotsEnabled: BooleanObject


class EnvironmentPropertyDescriptions(TypedDict, total=False):
    """Describes the execution properties for an Apache Flink runtime."""

    PropertyGroupDescriptions: Optional[PropertyGroups]


class ParallelismConfigurationDescription(TypedDict, total=False):
    """Describes parameters for how a Flink-based Kinesis Data Analytics
    application executes multiple tasks simultaneously.
    """

    ConfigurationType: Optional[ConfigurationType]
    Parallelism: Optional[Parallelism]
    ParallelismPerKPU: Optional[ParallelismPerKPU]
    CurrentParallelism: Optional[Parallelism]
    AutoScalingEnabled: Optional[BooleanObject]


class MonitoringConfigurationDescription(TypedDict, total=False):
    """Describes configuration parameters for CloudWatch logging for an
    application.
    """

    ConfigurationType: Optional[ConfigurationType]
    MetricsLevel: Optional[MetricsLevel]
    LogLevel: Optional[LogLevel]


class CheckpointConfigurationDescription(TypedDict, total=False):
    """Describes checkpointing parameters for a Flink-based Kinesis Data
    Analytics application.
    """

    ConfigurationType: Optional[ConfigurationType]
    CheckpointingEnabled: Optional[BooleanObject]
    CheckpointInterval: Optional[CheckpointInterval]
    MinPauseBetweenCheckpoints: Optional[MinPauseBetweenCheckpoints]


class FlinkApplicationConfigurationDescription(TypedDict, total=False):
    """Describes configuration parameters for a Flink-based Kinesis Data
    Analytics application.
    """

    CheckpointConfigurationDescription: Optional[CheckpointConfigurationDescription]
    MonitoringConfigurationDescription: Optional[MonitoringConfigurationDescription]
    ParallelismConfigurationDescription: Optional[ParallelismConfigurationDescription]
    JobPlanDescription: Optional[JobPlanDescription]


class FlinkRunConfiguration(TypedDict, total=False):
    """Describes the starting parameters for a Flink-based Kinesis Data
    Analytics application.
    """

    AllowNonRestoredState: Optional[BooleanObject]


class ApplicationRestoreConfiguration(TypedDict, total=False):
    """Specifies the method and snapshot to use when restarting an application
    using previously saved application state.
    """

    ApplicationRestoreType: ApplicationRestoreType
    SnapshotName: Optional[SnapshotName]


class RunConfigurationDescription(TypedDict, total=False):
    """Describes the starting properties for a Kinesis Data Analytics
    application.
    """

    ApplicationRestoreConfigurationDescription: Optional[ApplicationRestoreConfiguration]
    FlinkRunConfigurationDescription: Optional[FlinkRunConfiguration]


class SqlApplicationConfigurationDescription(TypedDict, total=False):
    """Describes the inputs, outputs, and reference data sources for a
    SQL-based Kinesis Data Analytics application.
    """

    InputDescriptions: Optional[InputDescriptions]
    OutputDescriptions: Optional[OutputDescriptions]
    ReferenceDataSourceDescriptions: Optional[ReferenceDataSourceDescriptions]


class ApplicationConfigurationDescription(TypedDict, total=False):
    """Describes details about the application code and starting parameters for
    a Kinesis Data Analytics application.
    """

    SqlApplicationConfigurationDescription: Optional[SqlApplicationConfigurationDescription]
    ApplicationCodeConfigurationDescription: Optional[ApplicationCodeConfigurationDescription]
    RunConfigurationDescription: Optional[RunConfigurationDescription]
    FlinkApplicationConfigurationDescription: Optional[FlinkApplicationConfigurationDescription]
    EnvironmentPropertyDescriptions: Optional[EnvironmentPropertyDescriptions]
    ApplicationSnapshotConfigurationDescription: Optional[
        ApplicationSnapshotConfigurationDescription
    ]
    VpcConfigurationDescriptions: Optional[VpcConfigurationDescriptions]
    ZeppelinApplicationConfigurationDescription: Optional[
        ZeppelinApplicationConfigurationDescription
    ]


class S3ContentBaseLocationUpdate(TypedDict, total=False):
    """The information required to update the S3 base location that holds the
    application.
    """

    BucketARNUpdate: Optional[BucketARN]
    BasePathUpdate: Optional[BasePath]


class DeployAsApplicationConfigurationUpdate(TypedDict, total=False):
    """Updates to the configuration information required to deploy an Amazon
    Data Analytics Studio notebook as an application with durable state.
    """

    S3ContentLocationUpdate: Optional[S3ContentBaseLocationUpdate]


class GlueDataCatalogConfigurationUpdate(TypedDict, total=False):
    """Updates to the configuration of the Glue Data Catalog that you use for
    SQL queries that you write in a Kinesis Data Analytics Studio notebook.
    """

    DatabaseARNUpdate: DatabaseARN


class CatalogConfigurationUpdate(TypedDict, total=False):
    """Updates to the configuration parameters for the default Amazon Glue
    database. You use this database for SQL queries that you write in a
    Kinesis Data Analytics Studio notebook.
    """

    GlueDataCatalogConfigurationUpdate: GlueDataCatalogConfigurationUpdate


class ZeppelinMonitoringConfigurationUpdate(TypedDict, total=False):
    """Updates to the monitoring configuration for Apache Zeppelin within a
    Kinesis Data Analytics Studio notebook.
    """

    LogLevelUpdate: LogLevel


class ZeppelinApplicationConfigurationUpdate(TypedDict, total=False):
    """Updates to the configuration of Kinesis Data Analytics Studio notebook."""

    MonitoringConfigurationUpdate: Optional[ZeppelinMonitoringConfigurationUpdate]
    CatalogConfigurationUpdate: Optional[CatalogConfigurationUpdate]
    DeployAsApplicationConfigurationUpdate: Optional[DeployAsApplicationConfigurationUpdate]
    CustomArtifactsConfigurationUpdate: Optional[CustomArtifactsConfigurationList]


class VpcConfigurationUpdate(TypedDict, total=False):
    """Describes updates to the VPC configuration used by the application."""

    VpcConfigurationId: Id
    SubnetIdUpdates: Optional[SubnetIds]
    SecurityGroupIdUpdates: Optional[SecurityGroupIds]


VpcConfigurationUpdates = List[VpcConfigurationUpdate]


class ApplicationSnapshotConfigurationUpdate(TypedDict, total=False):
    """Describes updates to whether snapshots are enabled for a Flink-based
    Kinesis Data Analytics application.
    """

    SnapshotsEnabledUpdate: BooleanObject


class EnvironmentPropertyUpdates(TypedDict, total=False):
    """Describes updates to the execution property groups for a Flink-based
    Kinesis Data Analytics application or a Studio notebook.
    """

    PropertyGroups: PropertyGroups


class ParallelismConfigurationUpdate(TypedDict, total=False):
    """Describes updates to parameters for how an application executes multiple
    tasks simultaneously.
    """

    ConfigurationTypeUpdate: Optional[ConfigurationType]
    ParallelismUpdate: Optional[Parallelism]
    ParallelismPerKPUUpdate: Optional[ParallelismPerKPU]
    AutoScalingEnabledUpdate: Optional[BooleanObject]


class MonitoringConfigurationUpdate(TypedDict, total=False):
    """Describes updates to configuration parameters for Amazon CloudWatch
    logging for an application.
    """

    ConfigurationTypeUpdate: Optional[ConfigurationType]
    MetricsLevelUpdate: Optional[MetricsLevel]
    LogLevelUpdate: Optional[LogLevel]


class CheckpointConfigurationUpdate(TypedDict, total=False):
    """Describes updates to the checkpointing parameters for a Flink-based
    Kinesis Data Analytics application.
    """

    ConfigurationTypeUpdate: Optional[ConfigurationType]
    CheckpointingEnabledUpdate: Optional[BooleanObject]
    CheckpointIntervalUpdate: Optional[CheckpointInterval]
    MinPauseBetweenCheckpointsUpdate: Optional[MinPauseBetweenCheckpoints]


class FlinkApplicationConfigurationUpdate(TypedDict, total=False):
    """Describes updates to the configuration parameters for a Flink-based
    Kinesis Data Analytics application.
    """

    CheckpointConfigurationUpdate: Optional[CheckpointConfigurationUpdate]
    MonitoringConfigurationUpdate: Optional[MonitoringConfigurationUpdate]
    ParallelismConfigurationUpdate: Optional[ParallelismConfigurationUpdate]


class S3ReferenceDataSourceUpdate(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes the Amazon
    S3 bucket name and object key name for an in-application reference
    table.
    """

    BucketARNUpdate: Optional[BucketARN]
    FileKeyUpdate: Optional[FileKey]


class ReferenceDataSourceUpdate(TypedDict, total=False):
    """When you update a reference data source configuration for a SQL-based
    Kinesis Data Analytics application, this object provides all the updated
    values (such as the source bucket name and object key name), the
    in-application table name that is created, and updated mapping
    information that maps the data in the Amazon S3 object to the
    in-application reference table that is created.
    """

    ReferenceId: Id
    TableNameUpdate: Optional[InAppTableName]
    S3ReferenceDataSourceUpdate: Optional[S3ReferenceDataSourceUpdate]
    ReferenceSchemaUpdate: Optional[SourceSchema]


ReferenceDataSourceUpdates = List[ReferenceDataSourceUpdate]


class LambdaOutputUpdate(TypedDict, total=False):
    """When you update an SQL-based Kinesis Data Analytics application's output
    configuration using the UpdateApplication operation, provides
    information about an Amazon Lambda function that is configured as the
    destination.
    """

    ResourceARNUpdate: ResourceARN


class KinesisFirehoseOutputUpdate(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, when updating an
    output configuration using the UpdateApplication operation, provides
    information about a Kinesis Data Firehose delivery stream that is
    configured as the destination.
    """

    ResourceARNUpdate: ResourceARN


class KinesisStreamsOutputUpdate(TypedDict, total=False):
    """When you update a SQL-based Kinesis Data Analytics application's output
    configuration using the UpdateApplication operation, provides
    information about a Kinesis data stream that is configured as the
    destination.
    """

    ResourceARNUpdate: ResourceARN


class OutputUpdate(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes updates to
    the output configuration identified by the ``OutputId``.
    """

    OutputId: Id
    NameUpdate: Optional[InAppStreamName]
    KinesisStreamsOutputUpdate: Optional[KinesisStreamsOutputUpdate]
    KinesisFirehoseOutputUpdate: Optional[KinesisFirehoseOutputUpdate]
    LambdaOutputUpdate: Optional[LambdaOutputUpdate]
    DestinationSchemaUpdate: Optional[DestinationSchema]


OutputUpdates = List[OutputUpdate]


class InputParallelismUpdate(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, provides updates to
    the parallelism count.
    """

    CountUpdate: InputParallelismCount


class InputSchemaUpdate(TypedDict, total=False):
    """Describes updates for an SQL-based Kinesis Data Analytics application's
    input schema.
    """

    RecordFormatUpdate: Optional[RecordFormat]
    RecordEncodingUpdate: Optional[RecordEncoding]
    RecordColumnUpdates: Optional[RecordColumns]


class KinesisFirehoseInputUpdate(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, when updating
    application input configuration, provides information about a Kinesis
    Data Firehose delivery stream as the streaming source.
    """

    ResourceARNUpdate: ResourceARN


class KinesisStreamsInputUpdate(TypedDict, total=False):
    """When you update the input configuration for a SQL-based Kinesis Data
    Analytics application, provides information about a Kinesis stream as
    the streaming source.
    """

    ResourceARNUpdate: ResourceARN


class InputLambdaProcessorUpdate(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, represents an update
    to the InputLambdaProcessor that is used to preprocess the records in
    the stream.
    """

    ResourceARNUpdate: ResourceARN


class InputProcessingConfigurationUpdate(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes updates to
    an InputProcessingConfiguration.
    """

    InputLambdaProcessorUpdate: InputLambdaProcessorUpdate


class InputUpdate(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, describes updates to
    a specific input configuration (identified by the ``InputId`` of an
    application).
    """

    InputId: Id
    NamePrefixUpdate: Optional[InAppStreamName]
    InputProcessingConfigurationUpdate: Optional[InputProcessingConfigurationUpdate]
    KinesisStreamsInputUpdate: Optional[KinesisStreamsInputUpdate]
    KinesisFirehoseInputUpdate: Optional[KinesisFirehoseInputUpdate]
    InputSchemaUpdate: Optional[InputSchemaUpdate]
    InputParallelismUpdate: Optional[InputParallelismUpdate]


InputUpdates = List[InputUpdate]


class SqlApplicationConfigurationUpdate(TypedDict, total=False):
    """Describes updates to the input streams, destination streams, and
    reference data sources for a SQL-based Kinesis Data Analytics
    application.
    """

    InputUpdates: Optional[InputUpdates]
    OutputUpdates: Optional[OutputUpdates]
    ReferenceDataSourceUpdates: Optional[ReferenceDataSourceUpdates]


class ApplicationConfigurationUpdate(TypedDict, total=False):
    """Describes updates to an application's configuration."""

    SqlApplicationConfigurationUpdate: Optional[SqlApplicationConfigurationUpdate]
    ApplicationCodeConfigurationUpdate: Optional[ApplicationCodeConfigurationUpdate]
    FlinkApplicationConfigurationUpdate: Optional[FlinkApplicationConfigurationUpdate]
    EnvironmentPropertyUpdates: Optional[EnvironmentPropertyUpdates]
    ApplicationSnapshotConfigurationUpdate: Optional[ApplicationSnapshotConfigurationUpdate]
    VpcConfigurationUpdates: Optional[VpcConfigurationUpdates]
    ZeppelinApplicationConfigurationUpdate: Optional[ZeppelinApplicationConfigurationUpdate]


class ApplicationMaintenanceConfigurationDescription(TypedDict, total=False):
    """The details of the maintenance configuration for the application."""

    ApplicationMaintenanceWindowStartTime: ApplicationMaintenanceWindowStartTime
    ApplicationMaintenanceWindowEndTime: ApplicationMaintenanceWindowEndTime


Timestamp = datetime


class ApplicationDetail(TypedDict, total=False):
    """Describes the application, including the application Amazon Resource
    Name (ARN), status, latest version, and input and output configurations.
    """

    ApplicationARN: ResourceARN
    ApplicationDescription: Optional[ApplicationDescription]
    ApplicationName: ApplicationName
    RuntimeEnvironment: RuntimeEnvironment
    ServiceExecutionRole: Optional[RoleARN]
    ApplicationStatus: ApplicationStatus
    ApplicationVersionId: ApplicationVersionId
    CreateTimestamp: Optional[Timestamp]
    LastUpdateTimestamp: Optional[Timestamp]
    ApplicationConfigurationDescription: Optional[ApplicationConfigurationDescription]
    CloudWatchLoggingOptionDescriptions: Optional[CloudWatchLoggingOptionDescriptions]
    ApplicationMaintenanceConfigurationDescription: Optional[
        ApplicationMaintenanceConfigurationDescription
    ]
    ApplicationVersionUpdatedFrom: Optional[ApplicationVersionId]
    ApplicationVersionRolledBackFrom: Optional[ApplicationVersionId]
    ConditionalToken: Optional[ConditionalToken]
    ApplicationVersionRolledBackTo: Optional[ApplicationVersionId]
    ApplicationMode: Optional[ApplicationMode]


class ApplicationMaintenanceConfigurationUpdate(TypedDict, total=False):
    """Describes the updated maintenance configuration for the application."""

    ApplicationMaintenanceWindowStartTimeUpdate: ApplicationMaintenanceWindowStartTime


class ApplicationSummary(TypedDict, total=False):
    """Provides application summary information, including the application
    Amazon Resource Name (ARN), name, and status.
    """

    ApplicationName: ApplicationName
    ApplicationARN: ResourceARN
    ApplicationStatus: ApplicationStatus
    ApplicationVersionId: ApplicationVersionId
    RuntimeEnvironment: RuntimeEnvironment
    ApplicationMode: Optional[ApplicationMode]


ApplicationSummaries = List[ApplicationSummary]


class ApplicationVersionSummary(TypedDict, total=False):
    """The summary of the application version."""

    ApplicationVersionId: ApplicationVersionId
    ApplicationStatus: ApplicationStatus


ApplicationVersionSummaries = List[ApplicationVersionSummary]


class CloudWatchLoggingOptionUpdate(TypedDict, total=False):
    """Describes the Amazon CloudWatch logging option updates."""

    CloudWatchLoggingOptionId: Id
    LogStreamARNUpdate: Optional[LogStreamARN]


CloudWatchLoggingOptionUpdates = List[CloudWatchLoggingOptionUpdate]
CloudWatchLoggingOptions = List[CloudWatchLoggingOption]
SessionExpirationDurationInSeconds = int


class CreateApplicationPresignedUrlRequest(ServiceRequest):
    ApplicationName: ApplicationName
    UrlType: UrlType
    SessionExpirationDurationInSeconds: Optional[SessionExpirationDurationInSeconds]


class CreateApplicationPresignedUrlResponse(TypedDict, total=False):
    AuthorizedUrl: Optional[AuthorizedUrl]


class Tag(TypedDict, total=False):
    """A key-value pair (the value is optional) that you can define and assign
    to Amazon resources. If you specify a tag that already exists, the tag
    value is replaced with the value that you specify in the request. Note
    that the maximum number of application tags includes system tags. The
    maximum number of user-defined application tags is 50. For more
    information, see `Using
    Tagging <https://docs.aws.amazon.com/kinesisanalytics/latest/java/how-tagging.html>`__.
    """

    Key: TagKey
    Value: Optional[TagValue]


Tags = List[Tag]


class CreateApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    ApplicationDescription: Optional[ApplicationDescription]
    RuntimeEnvironment: RuntimeEnvironment
    ServiceExecutionRole: RoleARN
    ApplicationConfiguration: Optional[ApplicationConfiguration]
    CloudWatchLoggingOptions: Optional[CloudWatchLoggingOptions]
    Tags: Optional[Tags]
    ApplicationMode: Optional[ApplicationMode]


class CreateApplicationResponse(TypedDict, total=False):
    ApplicationDetail: ApplicationDetail


class CreateApplicationSnapshotRequest(ServiceRequest):
    ApplicationName: ApplicationName
    SnapshotName: SnapshotName


class CreateApplicationSnapshotResponse(TypedDict, total=False):
    pass


class DeleteApplicationCloudWatchLoggingOptionRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: Optional[ApplicationVersionId]
    CloudWatchLoggingOptionId: Id
    ConditionalToken: Optional[ConditionalToken]


class DeleteApplicationCloudWatchLoggingOptionResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]
    CloudWatchLoggingOptionDescriptions: Optional[CloudWatchLoggingOptionDescriptions]


class DeleteApplicationInputProcessingConfigurationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    InputId: Id


class DeleteApplicationInputProcessingConfigurationResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]


class DeleteApplicationOutputRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    OutputId: Id


class DeleteApplicationOutputResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]


class DeleteApplicationReferenceDataSourceRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    ReferenceId: Id


class DeleteApplicationReferenceDataSourceResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]


class DeleteApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CreateTimestamp: Timestamp


class DeleteApplicationResponse(TypedDict, total=False):
    pass


class DeleteApplicationSnapshotRequest(ServiceRequest):
    ApplicationName: ApplicationName
    SnapshotName: SnapshotName
    SnapshotCreationTimestamp: Timestamp


class DeleteApplicationSnapshotResponse(TypedDict, total=False):
    pass


class DeleteApplicationVpcConfigurationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: Optional[ApplicationVersionId]
    VpcConfigurationId: Id
    ConditionalToken: Optional[ConditionalToken]


class DeleteApplicationVpcConfigurationResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationVersionId: Optional[ApplicationVersionId]


class DescribeApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    IncludeAdditionalDetails: Optional[BooleanObject]


class DescribeApplicationResponse(TypedDict, total=False):
    ApplicationDetail: ApplicationDetail


class DescribeApplicationSnapshotRequest(ServiceRequest):
    ApplicationName: ApplicationName
    SnapshotName: SnapshotName


class SnapshotDetails(TypedDict, total=False):
    """Provides details about a snapshot of application state."""

    SnapshotName: SnapshotName
    SnapshotStatus: SnapshotStatus
    ApplicationVersionId: ApplicationVersionId
    SnapshotCreationTimestamp: Optional[Timestamp]


class DescribeApplicationSnapshotResponse(TypedDict, total=False):
    SnapshotDetails: SnapshotDetails


class DescribeApplicationVersionRequest(ServiceRequest):
    ApplicationName: ApplicationName
    ApplicationVersionId: ApplicationVersionId


class DescribeApplicationVersionResponse(TypedDict, total=False):
    ApplicationVersionDetail: Optional[ApplicationDetail]


class S3Configuration(TypedDict, total=False):
    """For a SQL-based Kinesis Data Analytics application, provides a
    description of an Amazon S3 data source, including the Amazon Resource
    Name (ARN) of the S3 bucket and the name of the Amazon S3 object that
    contains the data.
    """

    BucketARN: BucketARN
    FileKey: FileKey


class DiscoverInputSchemaRequest(ServiceRequest):
    ResourceARN: Optional[ResourceARN]
    ServiceExecutionRole: RoleARN
    InputStartingPositionConfiguration: Optional[InputStartingPositionConfiguration]
    S3Configuration: Optional[S3Configuration]
    InputProcessingConfiguration: Optional[InputProcessingConfiguration]


ParsedInputRecord = List[ParsedInputRecordField]
ParsedInputRecords = List[ParsedInputRecord]


class DiscoverInputSchemaResponse(TypedDict, total=False):
    InputSchema: Optional[SourceSchema]
    ParsedInputRecords: Optional[ParsedInputRecords]
    ProcessedInputRecords: Optional[ProcessedInputRecords]
    RawInputRecords: Optional[RawInputRecords]


class ListApplicationSnapshotsRequest(ServiceRequest):
    ApplicationName: ApplicationName
    Limit: Optional[ListSnapshotsInputLimit]
    NextToken: Optional[NextToken]


SnapshotSummaries = List[SnapshotDetails]


class ListApplicationSnapshotsResponse(TypedDict, total=False):
    SnapshotSummaries: Optional[SnapshotSummaries]
    NextToken: Optional[NextToken]


class ListApplicationVersionsRequest(ServiceRequest):
    ApplicationName: ApplicationName
    Limit: Optional[ListApplicationVersionsInputLimit]
    NextToken: Optional[NextToken]


class ListApplicationVersionsResponse(TypedDict, total=False):
    ApplicationVersionSummaries: Optional[ApplicationVersionSummaries]
    NextToken: Optional[NextToken]


class ListApplicationsRequest(ServiceRequest):
    Limit: Optional[ListApplicationsInputLimit]
    NextToken: Optional[ApplicationName]


class ListApplicationsResponse(TypedDict, total=False):
    ApplicationSummaries: ApplicationSummaries
    NextToken: Optional[ApplicationName]


class ListTagsForResourceRequest(ServiceRequest):
    ResourceARN: KinesisAnalyticsARN


class ListTagsForResourceResponse(TypedDict, total=False):
    Tags: Optional[Tags]


class RollbackApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId


class RollbackApplicationResponse(TypedDict, total=False):
    ApplicationDetail: ApplicationDetail


class SqlRunConfiguration(TypedDict, total=False):
    """Describes the starting parameters for a SQL-based Kinesis Data Analytics
    application.
    """

    InputId: Id
    InputStartingPositionConfiguration: InputStartingPositionConfiguration


SqlRunConfigurations = List[SqlRunConfiguration]


class RunConfiguration(TypedDict, total=False):
    """Describes the starting parameters for an Kinesis Data Analytics
    application.
    """

    FlinkRunConfiguration: Optional[FlinkRunConfiguration]
    SqlRunConfigurations: Optional[SqlRunConfigurations]
    ApplicationRestoreConfiguration: Optional[ApplicationRestoreConfiguration]


class RunConfigurationUpdate(TypedDict, total=False):
    """Describes the updates to the starting parameters for a Kinesis Data
    Analytics application.
    """

    FlinkRunConfiguration: Optional[FlinkRunConfiguration]
    ApplicationRestoreConfiguration: Optional[ApplicationRestoreConfiguration]


class StartApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    RunConfiguration: Optional[RunConfiguration]


class StartApplicationResponse(TypedDict, total=False):
    pass


class StopApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    Force: Optional[BooleanObject]


class StopApplicationResponse(TypedDict, total=False):
    pass


TagKeys = List[TagKey]


class TagResourceRequest(ServiceRequest):
    ResourceARN: KinesisAnalyticsARN
    Tags: Tags


class TagResourceResponse(TypedDict, total=False):
    pass


class UntagResourceRequest(ServiceRequest):
    ResourceARN: KinesisAnalyticsARN
    TagKeys: TagKeys


class UntagResourceResponse(TypedDict, total=False):
    pass


class UpdateApplicationMaintenanceConfigurationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    ApplicationMaintenanceConfigurationUpdate: ApplicationMaintenanceConfigurationUpdate


class UpdateApplicationMaintenanceConfigurationResponse(TypedDict, total=False):
    ApplicationARN: Optional[ResourceARN]
    ApplicationMaintenanceConfigurationDescription: Optional[
        ApplicationMaintenanceConfigurationDescription
    ]


class UpdateApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: Optional[ApplicationVersionId]
    ApplicationConfigurationUpdate: Optional[ApplicationConfigurationUpdate]
    ServiceExecutionRoleUpdate: Optional[RoleARN]
    RunConfigurationUpdate: Optional[RunConfigurationUpdate]
    CloudWatchLoggingOptionUpdates: Optional[CloudWatchLoggingOptionUpdates]
    ConditionalToken: Optional[ConditionalToken]


class UpdateApplicationResponse(TypedDict, total=False):
    ApplicationDetail: ApplicationDetail


class Kinesisanalyticsv2Api:

    service = "kinesisanalyticsv2"
    version = "2018-05-23"

    @handler("AddApplicationCloudWatchLoggingOption")
    def add_application_cloud_watch_logging_option(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        cloud_watch_logging_option: CloudWatchLoggingOption,
        current_application_version_id: ApplicationVersionId = None,
        conditional_token: ConditionalToken = None,
    ) -> AddApplicationCloudWatchLoggingOptionResponse:
        """Adds an Amazon CloudWatch log stream to monitor application
        configuration errors.

        :param application_name: The Kinesis Data Analytics application name.
        :param cloud_watch_logging_option: Provides the Amazon CloudWatch log stream Amazon Resource Name (ARN).
        :param current_application_version_id: The version ID of the Kinesis Data Analytics application.
        :param conditional_token: A value you use to implement strong concurrency for application updates.
        :returns: AddApplicationCloudWatchLoggingOptionResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidRequestException:
        :raises InvalidApplicationConfigurationException:
        """
        raise NotImplementedError

    @handler("AddApplicationInput")
    def add_application_input(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        input: Input,
    ) -> AddApplicationInputResponse:
        """Adds a streaming source to your SQL-based Kinesis Data Analytics
        application.

        You can add a streaming source when you create an application, or you
        can use this operation to add a streaming source after you create an
        application. For more information, see CreateApplication.

        Any configuration update, including adding a streaming source using this
        operation, results in a new version of the application. You can use the
        DescribeApplication operation to find the current application version.

        :param application_name: The name of your existing application to which you want to add the
        streaming source.
        :param current_application_version_id: The current version of your application.
        :param input: The Input to add.
        :returns: AddApplicationInputResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises CodeValidationException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("AddApplicationInputProcessingConfiguration")
    def add_application_input_processing_configuration(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        input_id: Id,
        input_processing_configuration: InputProcessingConfiguration,
    ) -> AddApplicationInputProcessingConfigurationResponse:
        """Adds an InputProcessingConfiguration to a SQL-based Kinesis Data
        Analytics application. An input processor pre-processes records on the
        input stream before the application's SQL code executes. Currently, the
        only input processor available is `Amazon
        Lambda <https://docs.aws.amazon.com/lambda/>`__.

        :param application_name: The name of the application to which you want to add the input
        processing configuration.
        :param current_application_version_id: The version of the application to which you want to add the input
        processing configuration.
        :param input_id: The ID of the input configuration to add the input processing
        configuration to.
        :param input_processing_configuration: The InputProcessingConfiguration to add to the application.
        :returns: AddApplicationInputProcessingConfigurationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("AddApplicationOutput")
    def add_application_output(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        output: Output,
    ) -> AddApplicationOutputResponse:
        """Adds an external destination to your SQL-based Kinesis Data Analytics
        application.

        If you want Kinesis Data Analytics to deliver data from an
        in-application stream within your application to an external destination
        (such as an Kinesis data stream, a Kinesis Data Firehose delivery
        stream, or an Amazon Lambda function), you add the relevant
        configuration to your application using this operation. You can
        configure one or more outputs for your application. Each output
        configuration maps an in-application stream and an external destination.

        You can use one of the output configurations to deliver data from your
        in-application error stream to an external destination so that you can
        analyze the errors.

        Any configuration update, including adding a streaming source using this
        operation, results in a new version of the application. You can use the
        DescribeApplication operation to find the current application version.

        :param application_name: The name of the application to which you want to add the output
        configuration.
        :param current_application_version_id: The version of the application to which you want to add the output
        configuration.
        :param output: An array of objects, each describing one output configuration.
        :returns: AddApplicationOutputResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("AddApplicationReferenceDataSource")
    def add_application_reference_data_source(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        reference_data_source: ReferenceDataSource,
    ) -> AddApplicationReferenceDataSourceResponse:
        """Adds a reference data source to an existing SQL-based Kinesis Data
        Analytics application.

        Kinesis Data Analytics reads reference data (that is, an Amazon S3
        object) and creates an in-application table within your application. In
        the request, you provide the source (S3 bucket name and object key
        name), name of the in-application table to create, and the necessary
        mapping information that describes how data in an Amazon S3 object maps
        to columns in the resulting in-application table.

        :param application_name: The name of an existing application.
        :param current_application_version_id: The version of the application for which you are adding the reference
        data source.
        :param reference_data_source: The reference data source can be an object in your Amazon S3 bucket.
        :returns: AddApplicationReferenceDataSourceResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("AddApplicationVpcConfiguration")
    def add_application_vpc_configuration(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        vpc_configuration: VpcConfiguration,
        current_application_version_id: ApplicationVersionId = None,
        conditional_token: ConditionalToken = None,
    ) -> AddApplicationVpcConfigurationResponse:
        """Adds a Virtual Private Cloud (VPC) configuration to the application.
        Applications can use VPCs to store and access resources securely.

        Note the following about VPC configurations for Kinesis Data Analytics
        applications:

        -  VPC configurations are not supported for SQL applications.

        -  When a VPC is added to a Kinesis Data Analytics application, the
           application can no longer be accessed from the Internet directly. To
           enable Internet access to the application, add an Internet gateway to
           your VPC.

        :param application_name: The name of an existing application.
        :param vpc_configuration: Description of the VPC to add to the application.
        :param current_application_version_id: The version of the application to which you want to add the VPC
        configuration.
        :param conditional_token: A value you use to implement strong concurrency for application updates.
        :returns: AddApplicationVpcConfigurationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidApplicationConfigurationException:
        """
        raise NotImplementedError

    @handler("CreateApplication")
    def create_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        runtime_environment: RuntimeEnvironment,
        service_execution_role: RoleARN,
        application_description: ApplicationDescription = None,
        application_configuration: ApplicationConfiguration = None,
        cloud_watch_logging_options: CloudWatchLoggingOptions = None,
        tags: Tags = None,
        application_mode: ApplicationMode = None,
    ) -> CreateApplicationResponse:
        """Creates a Kinesis Data Analytics application. For information about
        creating a Kinesis Data Analytics application, see `Creating an
        Application <https://docs.aws.amazon.com/kinesisanalytics/latest/java/getting-started.html>`__.

        :param application_name: The name of your application (for example, ``sample-app``).
        :param runtime_environment: The runtime environment for the application.
        :param service_execution_role: The IAM role used by the application to access Kinesis data streams,
        Kinesis Data Firehose delivery streams, Amazon S3 objects, and other
        external resources.
        :param application_description: A summary description of the application.
        :param application_configuration: Use this parameter to configure the application.
        :param cloud_watch_logging_options: Use this parameter to configure an Amazon CloudWatch log stream to
        monitor application configuration errors.
        :param tags: A list of one or more tags to assign to the application.
        :param application_mode: Use the ``STREAMING`` mode to create a Kinesis Data Analytics For Flink
        application.
        :returns: CreateApplicationResponse
        :raises CodeValidationException:
        :raises ResourceInUseException:
        :raises LimitExceededException:
        :raises InvalidArgumentException:
        :raises InvalidRequestException:
        :raises TooManyTagsException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("CreateApplicationPresignedUrl")
    def create_application_presigned_url(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        url_type: UrlType,
        session_expiration_duration_in_seconds: SessionExpirationDurationInSeconds = None,
    ) -> CreateApplicationPresignedUrlResponse:
        """Creates and returns a URL that you can use to connect to an
        application's extension.

        The IAM role or user used to call this API defines the permissions to
        access the extension. After the presigned URL is created, no additional
        permission is required to access this URL. IAM authorization policies
        for this API are also enforced for every HTTP request that attempts to
        connect to the extension.

        You control the amount of time that the URL will be valid using the
        ``SessionExpirationDurationInSeconds`` parameter. If you do not provide
        this parameter, the returned URL is valid for twelve hours.

        The URL that you get from a call to CreateApplicationPresignedUrl must
        be used within 3 minutes to be valid. If you first try to use the URL
        after the 3-minute limit expires, the service returns an HTTP 403
        Forbidden error.

        :param application_name: The name of the application.
        :param url_type: The type of the extension for which to create and return a URL.
        :param session_expiration_duration_in_seconds: The duration in seconds for which the returned URL will be valid.
        :returns: CreateApplicationPresignedUrlResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        """
        raise NotImplementedError

    @handler("CreateApplicationSnapshot")
    def create_application_snapshot(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        snapshot_name: SnapshotName,
    ) -> CreateApplicationSnapshotResponse:
        """Creates a snapshot of the application's state data.

        :param application_name: The name of an existing application.
        :param snapshot_name: An identifier for the application snapshot.
        :returns: CreateApplicationSnapshotResponse
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises LimitExceededException:
        :raises InvalidArgumentException:
        :raises UnsupportedOperationException:
        :raises InvalidRequestException:
        :raises InvalidApplicationConfigurationException:
        """
        raise NotImplementedError

    @handler("DeleteApplication")
    def delete_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        create_timestamp: Timestamp,
    ) -> DeleteApplicationResponse:
        """Deletes the specified application. Kinesis Data Analytics halts
        application execution and deletes the application.

        :param application_name: The name of the application to delete.
        :param create_timestamp: Use the ``DescribeApplication`` operation to get this value.
        :returns: DeleteApplicationResponse
        :raises ConcurrentModificationException:
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises InvalidRequestException:
        :raises InvalidApplicationConfigurationException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationCloudWatchLoggingOption")
    def delete_application_cloud_watch_logging_option(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        cloud_watch_logging_option_id: Id,
        current_application_version_id: ApplicationVersionId = None,
        conditional_token: ConditionalToken = None,
    ) -> DeleteApplicationCloudWatchLoggingOptionResponse:
        """Deletes an Amazon CloudWatch log stream from an Kinesis Data Analytics
        application.

        :param application_name: The application name.
        :param cloud_watch_logging_option_id: The ``CloudWatchLoggingOptionId`` of the Amazon CloudWatch logging
        option to delete.
        :param current_application_version_id: The version ID of the application.
        :param conditional_token: A value you use to implement strong concurrency for application updates.
        :returns: DeleteApplicationCloudWatchLoggingOptionResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidRequestException:
        :raises InvalidApplicationConfigurationException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationInputProcessingConfiguration")
    def delete_application_input_processing_configuration(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        input_id: Id,
    ) -> DeleteApplicationInputProcessingConfigurationResponse:
        """Deletes an InputProcessingConfiguration from an input.

        :param application_name: The name of the application.
        :param current_application_version_id: The application version.
        :param input_id: The ID of the input configuration from which to delete the input
        processing configuration.
        :returns: DeleteApplicationInputProcessingConfigurationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationOutput")
    def delete_application_output(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        output_id: Id,
    ) -> DeleteApplicationOutputResponse:
        """Deletes the output destination configuration from your SQL-based Kinesis
        Data Analytics application's configuration. Kinesis Data Analytics will
        no longer write data from the corresponding in-application stream to the
        external output destination.

        :param application_name: The application name.
        :param current_application_version_id: The application version.
        :param output_id: The ID of the configuration to delete.
        :returns: DeleteApplicationOutputResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationReferenceDataSource")
    def delete_application_reference_data_source(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        reference_id: Id,
    ) -> DeleteApplicationReferenceDataSourceResponse:
        """Deletes a reference data source configuration from the specified
        SQL-based Kinesis Data Analytics application's configuration.

        If the application is running, Kinesis Data Analytics immediately
        removes the in-application table that you created using the
        AddApplicationReferenceDataSource operation.

        :param application_name: The name of an existing application.
        :param current_application_version_id: The current application version.
        :param reference_id: The ID of the reference data source.
        :returns: DeleteApplicationReferenceDataSourceResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationSnapshot")
    def delete_application_snapshot(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        snapshot_name: SnapshotName,
        snapshot_creation_timestamp: Timestamp,
    ) -> DeleteApplicationSnapshotResponse:
        """Deletes a snapshot of application state.

        :param application_name: The name of an existing application.
        :param snapshot_name: The identifier for the snapshot delete.
        :param snapshot_creation_timestamp: The creation timestamp of the application snapshot to delete.
        :returns: DeleteApplicationSnapshotResponse
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises UnsupportedOperationException:
        :raises InvalidRequestException:
        :raises ResourceNotFoundException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationVpcConfiguration")
    def delete_application_vpc_configuration(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        vpc_configuration_id: Id,
        current_application_version_id: ApplicationVersionId = None,
        conditional_token: ConditionalToken = None,
    ) -> DeleteApplicationVpcConfigurationResponse:
        """Removes a VPC configuration from a Kinesis Data Analytics application.

        :param application_name: The name of an existing application.
        :param vpc_configuration_id: The ID of the VPC configuration to delete.
        :param current_application_version_id: The current application version ID.
        :param conditional_token: A value you use to implement strong concurrency for application updates.
        :returns: DeleteApplicationVpcConfigurationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidApplicationConfigurationException:
        """
        raise NotImplementedError

    @handler("DescribeApplication")
    def describe_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        include_additional_details: BooleanObject = None,
    ) -> DescribeApplicationResponse:
        """Returns information about a specific Kinesis Data Analytics application.

        If you want to retrieve a list of all applications in your account, use
        the ListApplications operation.

        :param application_name: The name of the application.
        :param include_additional_details: Displays verbose information about a Kinesis Data Analytics application,
        including the application's job plan.
        :returns: DescribeApplicationResponse
        :raises ResourceNotFoundException:
        :raises InvalidArgumentException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("DescribeApplicationSnapshot")
    def describe_application_snapshot(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        snapshot_name: SnapshotName,
    ) -> DescribeApplicationSnapshotResponse:
        """Returns information about a snapshot of application state data.

        :param application_name: The name of an existing application.
        :param snapshot_name: The identifier of an application snapshot.
        :returns: DescribeApplicationSnapshotResponse
        :raises ResourceNotFoundException:
        :raises InvalidArgumentException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("DescribeApplicationVersion")
    def describe_application_version(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        application_version_id: ApplicationVersionId,
    ) -> DescribeApplicationVersionResponse:
        """Provides a detailed description of a specified version of the
        application. To see a list of all the versions of an application, invoke
        the ListApplicationVersions operation.

        This operation is supported only for Amazon Kinesis Data Analytics for
        Apache Flink.

        :param application_name: The name of the application for which you want to get the version
        description.
        :param application_version_id: The ID of the application version for which you want to get the
        description.
        :returns: DescribeApplicationVersionResponse
        :raises InvalidArgumentException:
        :raises ResourceNotFoundException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("DiscoverInputSchema")
    def discover_input_schema(
        self,
        context: RequestContext,
        service_execution_role: RoleARN,
        resource_arn: ResourceARN = None,
        input_starting_position_configuration: InputStartingPositionConfiguration = None,
        s3_configuration: S3Configuration = None,
        input_processing_configuration: InputProcessingConfiguration = None,
    ) -> DiscoverInputSchemaResponse:
        """Infers a schema for a SQL-based Kinesis Data Analytics application by
        evaluating sample records on the specified streaming source (Kinesis
        data stream or Kinesis Data Firehose delivery stream) or Amazon S3
        object. In the response, the operation returns the inferred schema and
        also the sample records that the operation used to infer the schema.

        You can use the inferred schema when configuring a streaming source for
        your application. When you create an application using the Kinesis Data
        Analytics console, the console uses this operation to infer a schema and
        show it in the console user interface.

        :param service_execution_role: The ARN of the role that is used to access the streaming source.
        :param resource_arn: The Amazon Resource Name (ARN) of the streaming source.
        :param input_starting_position_configuration: The point at which you want Kinesis Data Analytics to start reading
        records from the specified streaming source discovery purposes.
        :param s3_configuration: Specify this parameter to discover a schema from data in an Amazon S3
        object.
        :param input_processing_configuration: The InputProcessingConfiguration to use to preprocess the records before
        discovering the schema of the records.
        :returns: DiscoverInputSchemaResponse
        :raises InvalidArgumentException:
        :raises UnableToDetectSchemaException:
        :raises ResourceProvisionedThroughputExceededException:
        :raises ServiceUnavailableException:
        :raises InvalidRequestException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("ListApplicationSnapshots")
    def list_application_snapshots(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        limit: ListSnapshotsInputLimit = None,
        next_token: NextToken = None,
    ) -> ListApplicationSnapshotsResponse:
        """Lists information about the current application snapshots.

        :param application_name: The name of an existing application.
        :param limit: The maximum number of application snapshots to list.
        :param next_token: Use this parameter if you receive a ``NextToken`` response in a previous
        request that indicates that there is more output available.
        :returns: ListApplicationSnapshotsResponse
        :raises InvalidArgumentException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("ListApplicationVersions")
    def list_application_versions(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        limit: ListApplicationVersionsInputLimit = None,
        next_token: NextToken = None,
    ) -> ListApplicationVersionsResponse:
        """Lists all the versions for the specified application, including versions
        that were rolled back. The response also includes a summary of the
        configuration associated with each version.

        To get the complete description of a specific application version,
        invoke the DescribeApplicationVersion operation.

        This operation is supported only for Amazon Kinesis Data Analytics for
        Apache Flink.

        :param application_name: The name of the application for which you want to list all versions.
        :param limit: The maximum number of versions to list in this invocation of the
        operation.
        :param next_token: If a previous invocation of this operation returned a pagination token,
        pass it into this value to retrieve the next set of results.
        :returns: ListApplicationVersionsResponse
        :raises InvalidArgumentException:
        :raises ResourceNotFoundException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("ListApplications")
    def list_applications(
        self,
        context: RequestContext,
        limit: ListApplicationsInputLimit = None,
        next_token: ApplicationName = None,
    ) -> ListApplicationsResponse:
        """Returns a list of Kinesis Data Analytics applications in your account.
        For each application, the response includes the application name, Amazon
        Resource Name (ARN), and status.

        If you want detailed information about a specific application, use
        DescribeApplication.

        :param limit: The maximum number of applications to list.
        :param next_token: If a previous command returned a pagination token, pass it into this
        value to retrieve the next set of results.
        :returns: ListApplicationsResponse
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(
        self, context: RequestContext, resource_arn: KinesisAnalyticsARN
    ) -> ListTagsForResourceResponse:
        """Retrieves the list of key-value tags assigned to the application. For
        more information, see `Using
        Tagging <https://docs.aws.amazon.com/kinesisanalytics/latest/java/how-tagging.html>`__.

        :param resource_arn: The ARN of the application for which to retrieve tags.
        :returns: ListTagsForResourceResponse
        :raises ResourceNotFoundException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("RollbackApplication")
    def rollback_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
    ) -> RollbackApplicationResponse:
        """Reverts the application to the previous running version. You can roll
        back an application if you suspect it is stuck in a transient status.

        You can roll back an application only if it is in the ``UPDATING`` or
        ``AUTOSCALING`` status.

        When you rollback an application, it loads state data from the last
        successful snapshot. If the application has no snapshots, Kinesis Data
        Analytics rejects the rollback request.

        This action is not supported for Kinesis Data Analytics for SQL
        applications.

        :param application_name: The name of the application.
        :param current_application_version_id: The current application version ID.
        :returns: RollbackApplicationResponse
        :raises ResourceNotFoundException:
        :raises InvalidArgumentException:
        :raises ResourceInUseException:
        :raises InvalidRequestException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("StartApplication")
    def start_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        run_configuration: RunConfiguration = None,
    ) -> StartApplicationResponse:
        """Starts the specified Kinesis Data Analytics application. After creating
        an application, you must exclusively call this operation to start your
        application.

        :param application_name: The name of the application.
        :param run_configuration: Identifies the run configuration (start parameters) of a Kinesis Data
        Analytics application.
        :returns: StartApplicationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises InvalidApplicationConfigurationException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("StopApplication")
    def stop_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        force: BooleanObject = None,
    ) -> StopApplicationResponse:
        """Stops the application from processing data. You can stop an application
        only if it is in the running status, unless you set the ``Force``
        parameter to ``true``.

        You can use the DescribeApplication operation to find the application
        status.

        Kinesis Data Analytics takes a snapshot when the application is stopped,
        unless ``Force`` is set to ``true``.

        :param application_name: The name of the running application to stop.
        :param force: Set to ``true`` to force the application to stop.
        :returns: StopApplicationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises InvalidRequestException:
        :raises InvalidApplicationConfigurationException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(
        self, context: RequestContext, resource_arn: KinesisAnalyticsARN, tags: Tags
    ) -> TagResourceResponse:
        """Adds one or more key-value tags to a Kinesis Data Analytics application.
        Note that the maximum number of application tags includes system tags.
        The maximum number of user-defined application tags is 50. For more
        information, see `Using
        Tagging <https://docs.aws.amazon.com/kinesisanalytics/latest/java/how-tagging.html>`__.

        :param resource_arn: The ARN of the application to assign the tags.
        :param tags: The key-value tags to assign to the application.
        :returns: TagResourceResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises TooManyTagsException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_arn: KinesisAnalyticsARN, tag_keys: TagKeys
    ) -> UntagResourceResponse:
        """Removes one or more tags from a Kinesis Data Analytics application. For
        more information, see `Using
        Tagging <https://docs.aws.amazon.com/kinesisanalytics/latest/java/how-tagging.html>`__.

        :param resource_arn: The ARN of the Kinesis Data Analytics application from which to remove
        the tags.
        :param tag_keys: A list of keys of tags to remove from the specified application.
        :returns: UntagResourceResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises TooManyTagsException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("UpdateApplication")
    def update_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId = None,
        application_configuration_update: ApplicationConfigurationUpdate = None,
        service_execution_role_update: RoleARN = None,
        run_configuration_update: RunConfigurationUpdate = None,
        cloud_watch_logging_option_updates: CloudWatchLoggingOptionUpdates = None,
        conditional_token: ConditionalToken = None,
    ) -> UpdateApplicationResponse:
        """Updates an existing Kinesis Data Analytics application. Using this
        operation, you can update application code, input configuration, and
        output configuration.

        Kinesis Data Analytics updates the ``ApplicationVersionId`` each time
        you update your application.

        You cannot update the ``RuntimeEnvironment`` of an existing application.
        If you need to update an application's ``RuntimeEnvironment``, you must
        delete the application and create it again.

        :param application_name: The name of the application to update.
        :param current_application_version_id: The current application version ID.
        :param application_configuration_update: Describes application configuration updates.
        :param service_execution_role_update: Describes updates to the service execution role.
        :param run_configuration_update: Describes updates to the application's starting parameters.
        :param cloud_watch_logging_option_updates: Describes application Amazon CloudWatch logging option updates.
        :param conditional_token: A value you use to implement strong concurrency for application updates.
        :returns: UpdateApplicationResponse
        :raises CodeValidationException:
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises InvalidRequestException:
        :raises InvalidApplicationConfigurationException:
        :raises LimitExceededException:
        """
        raise NotImplementedError

    @handler("UpdateApplicationMaintenanceConfiguration")
    def update_application_maintenance_configuration(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        application_maintenance_configuration_update: ApplicationMaintenanceConfigurationUpdate,
    ) -> UpdateApplicationMaintenanceConfigurationResponse:
        """Updates the maintenance configuration of the Kinesis Data Analytics
        application.

        You can invoke this operation on an application that is in one of the
        two following states: ``READY`` or ``RUNNING``. If you invoke it when
        the application is in a state other than these two states, it throws a
        ``ResourceInUseException``. The service makes use of the updated
        configuration the next time it schedules maintenance for the
        application. If you invoke this operation after the service schedules
        maintenance, the service will apply the configuration update the next
        time it schedules maintenance for the application. This means that you
        might not see the maintenance configuration update applied to the
        maintenance process that follows a successful invocation of this
        operation, but to the following maintenance process instead.

        To see the current maintenance configuration of your application, invoke
        the DescribeApplication operation.

        For information about application maintenance, see `Kinesis Data
        Analytics for Apache Flink
        Maintenance <https://docs.aws.amazon.com/kinesisanalytics/latest/java/maintenance.html>`__.

        This operation is supported only for Amazon Kinesis Data Analytics for
        Apache Flink.

        :param application_name: The name of the application for which you want to update the maintenance
        configuration.
        :param application_maintenance_configuration_update: Describes the application maintenance configuration update.
        :returns: UpdateApplicationMaintenanceConfigurationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError



_B='specification'
_A='latest'
import glob,os
from typing import List
from localstack.packages import InstallTarget,Package,PackageInstaller
from localstack.utils.archives import download_and_extract_with_retry
from localstack.utils.files import mkdir,rm_rf
from localstack.utils.http import download
DOWNLOAD_FULL_SPECS=False
SPEC_ZIP_URL='https://github.com/Azure/azure-rest-api-specs/archive/main.zip'
GITHUB_AZURE_SPECS_URL='https://raw.githubusercontent.com/Azure/azure-rest-api-specs/<branch>/'
SPEC_FILES={}
SPECS={'eventhub-hubs':'eventhub/resource-manager/Microsoft.EventHub/stable/2017-04-01/eventhubs.json','eventhub-namespaces':'eventhub/resource-manager/Microsoft.EventHub/stable/2017-04-01/namespaces.json','eventhub-authrules':'eventhub/resource-manager/Microsoft.EventHub/stable/2017-04-01/AuthorizationRules.json','eventhub-consumergroups':'eventhub/resource-manager/Microsoft.EventHub/stable/2017-04-01/consumergroups.json','storage-accounts':'storage/resource-manager/Microsoft.Storage/stable/2019-06-01/storage.json','storage-blob':'storage/data-plane/Microsoft.BlobStorage/preview/2021-02-12/blob.json','storage-queue':'storage/data-plane/Microsoft.QueueStorage/preview/2018-03-28/queue.json','subscriptions':'subscription/resource-manager/Microsoft.Subscription/stable/2020-09-01/subscriptions.json','web-apps':'web/resource-manager/Microsoft.Web/stable/2020-06-01/WebApps.json','web-resources':'web/resource-manager/Microsoft.Web/stable/2020-06-01/ResourceProvider.json','service-plans':'web/resource-manager/Microsoft.Web/stable/2020-06-01/AppServicePlans.json','resource-groups':'resources/resource-manager/Microsoft.Resources/stable/2020-10-01/resources.json','resource-graph':'resourcegraph/resource-manager/Microsoft.ResourceGraph/stable/2019-04-01/resourcegraph.json','appinsights-components':'applicationinsights/resource-manager/Microsoft.Insights/stable/2015-05-01/components_API.json','devops':'devops/resource-manager/Microsoft.DevOps/preview/2020-07-13-preview/devops.json','servicebus':'servicebus/resource-manager/Microsoft.ServiceBus/stable/2017-04-01/eventhubs.json'}
class AzureApiSpecsPackage(Package):
	def __init__(A):super().__init__('Azure Rest API Specs',_A)
	def get_versions(A):return[_A]
	def _get_installer(A,version):return AzureApiSpecsPackageInstaller()
class AzureApiSpecsPackageInstaller(PackageInstaller):
	def __init__(A):super().__init__('azure-api-specs',_A)
	def _get_install_dir(A,target):return os.path.join(target.value,A.name,'azure-rest-api-specs-main')
	def _get_install_marker_path(A,install_dir):return os.path.join(install_dir,_B)
	def _install(H,target):
		B=H._get_install_dir(target);mkdir(B);D=os.path.realpath(os.path.join(B,'..'));A=os.path.join(B,'package.json')
		if DOWNLOAD_FULL_SPECS and not os.path.exists(A):E=os.path.join(D,'tmp.azure.specs.zip');download_and_extract_with_retry(SPEC_ZIP_URL,E,D);rm_rf(E)
		A=os.path.join(B,_B,'eventgrid','resource-manager','Microsoft.EventGrid','preview')
		if os.path.exists(A):
			for I in glob.glob('%s/**/preview/'%D,recursive=True):rm_rf(I)
		if not SPEC_FILES:
			for (L,C) in SPECS.items():
				F='main'
				if isinstance(C,dict):F,C=next(iter(C.items()))
				SPEC_FILES.update({C:GITHUB_AZURE_SPECS_URL.replace('<branch>',F)})
		for (G,J) in SPEC_FILES.items():
			A=os.path.join(B,_B,G)
			if not os.path.exists(A):K='%s/specification/%s'%(J,G);download(K,A)
azure_api_specs_package=AzureApiSpecsPackage()


_B=False
_A=None
import copy,json,logging,sys
from abc import ABC
from datetime import datetime
from enum import Enum
from typing import Any,Dict,List,Optional,Set,Union
if sys.version_info>=(3,8):from typing import Final
else:from typing_extensions import Final
import yaml
from localstack_ext.bootstrap.pods.constants import NIL_PTR
from localstack_ext.bootstrap.state_utils import DYNAMODB_DIR,KINESIS_DIR
from localstack_ext.constants import API_STATES_DIR
LOG=logging.getLogger(__name__)
class SerializableObject(ABC):
	def to_dict(A):return copy.deepcopy(A.__dict__)
	@classmethod
	def from_dict(B,obj):A=B();A.__dict__.update(obj);return A
	def to_yaml(A):return yaml.dump(A.to_dict())
	def to_json(A):return json.dumps(A.to_dict())
	def __str__(A):return A.to_yaml()
	def __repr__(A):return A.to_json()
class PodObject(SerializableObject):
	def __init__(A,hash_ref):A.hash_ref=hash_ref;A.obj_type=A.__class__.__name__
class Serialization(Enum):MAIN=API_STATES_DIR;DDB=DYNAMODB_DIR;KINESIS=KINESIS_DIR;serializer_root_lookup={str(MAIN):API_STATES_DIR,str(DDB):DYNAMODB_DIR,str(KINESIS):KINESIS_DIR}
class StateFileRef(PodObject):
	def __init__(A,hash_ref=_A,rel_path=_A,file_name=_A,size=_A,service=_A,region=_A,account_id=_A,serialization=_A):super(StateFileRef,A).__init__(hash_ref);A.rel_path=rel_path;A.file_name=file_name;A.size=size;A.service=service;A.region=region;A.account_id=account_id;A.serialization=serialization
	def __eq__(A,other):
		B=other
		if not B:return _B
		if not isinstance(B,StateFileRef):return _B
		return A.hash_ref==B.hash_ref and A.account_id==B.account_id and A.region==B.region and A.service==A.service and A.file_name==B.file_name and A.size==B.size
	def __hash__(A):return hash((A.hash_ref,A.account_id,A.region,A.service,A.file_name,A.size))
	def congruent(A,other):
		B=other
		if not B:return _B
		if not isinstance(B,StateFileRef):return _B
		return A.region==B.region and A.account_id==A.account_id and A.service==B.service and A.file_name==B.file_name and A.rel_path==B.rel_path
	def any_congruence(A,others):
		for B in others:
			if A.congruent(B):return True
		return _B
	def to_dict(C):B='serialization';A=super().to_dict();A[B]=A[B].name;return A
	@classmethod
	def from_dict(B,obj):A=super().from_dict(obj);A.serialization=Serialization[A.serialization];return A
class PodNode(PodObject):
	def __init__(A,hash_ref,state_files,parent_ptr):super(PodNode,A).__init__(hash_ref);A.state_files=state_files;A.parent_ptr=parent_ptr
	def state_files_info(A):return '\n'.join(list(map(lambda state_file:str(state_file),A.state_files)))
	def to_dict(C):
		B='state_files';A=super().to_dict()
		if isinstance(A.get(B),set):A[B]=list((C.to_dict()for C in A[B]))
		return A
	@classmethod
	def from_dict(B,obj):A=super().from_dict(obj);A.state_files=set((StateFileRef.from_dict(B)for B in A.state_files or[]));return A
class Commit(SerializableObject):
	tail_ptr:str;head_ptr:str;message:str;timestamp:str
	def __init__(A,tail_ptr=_A,head_ptr=_A,message=_A,timestamp=_A):A.tail_ptr=tail_ptr;A.head_ptr=head_ptr;A.message=message;A.timestamp=timestamp or str(datetime.now().timestamp())
	def get_summary(A):return f"from {A.tail_ptr}, to {A.head_ptr}, message: {A.message}, time: {A.timestamp}"
class Revision(PodNode):
	DEFAULT_INITIAL_REVISION_NUMBER=1
	def __init__(A,hash_ref=_A,state_files=_A,parent_ptr=_A,creator=_A,rid=_A,revision_number=_A,assoc_commit=_A,metamodel_file=_A):super(Revision,A).__init__(hash_ref,state_files,parent_ptr);A.creator=creator;A.rid=rid;A.revision_number=revision_number or A.DEFAULT_INITIAL_REVISION_NUMBER;A.assoc_commit=assoc_commit;A.metamodel_file=metamodel_file or''
	def to_dict(A):
		B=super().to_dict()
		if A.assoc_commit:B['assoc_commit']=A.assoc_commit.to_dict()
		return B
	@classmethod
	def from_dict(B,obj):
		A=super().from_dict(obj)
		if A.assoc_commit:A.assoc_commit=Commit.from_dict(A.assoc_commit)
		return A
class Version(PodNode):
	DEFAULT_INITIAL_VERSION_NUMBER=1;creator:str;comment:str;revisions:List[Revision];outgoing_revision_ptrs:Set[str];incoming_revision_ptr:str;version_number:int
	def __init__(A,hash_ref=_A,state_files=_A,parent_ptr=_A,creator=_A,comment=_A,outgoing_revision_ptrs=_A,incoming_revision_ptr=_A,version_number=_A):super(Version,A).__init__(hash_ref,state_files,parent_ptr);A.creator=creator;A.comment=comment;A.outgoing_revision_ptrs=outgoing_revision_ptrs;A.incoming_revision_ptr=incoming_revision_ptr;A.version_number=version_number;A.revisions=[]
	def get_latest_revision(A,with_commit=_B):
		B=A.revisions_with_commit if with_commit else A.revisions
		if not B:raise Exception('No revisions present in cloud pod version')
		return B[-1]
	@property
	def revisions_with_commit(self):return[A for A in self.revisions if A.assoc_commit]
	@property
	def active_revision_ptr(self):
		A=self.get_latest_revision()
		if not A:return NIL_PTR
		return A.hash_ref
	def get_revision(B,key):
		for A in B.revisions:
			if key in(A.hash_ref,A.revision_number):return A
	def update_revision_key(A,old_key,new_key):
		B=old_key
		if B in A.outgoing_revision_ptrs:A.outgoing_revision_ptrs.remove(B)
		A.outgoing_revision_ptrs.add(new_key)
	def to_dict(D):C='revisions';B='outgoing_revision_ptrs';A=super().to_dict();A[B]=list(A[B]or[]);A[C]=list((B.to_dict()for B in A[C]or[]));return A
	@classmethod
	def from_dict(B,obj):A=super().from_dict(obj);A.outgoing_revision_ptrs=set(A.outgoing_revision_ptrs or[]);A.revisions=[Revision.from_dict(B)for B in A.revisions];return A


_A='install'
import functools,logging,os,threading
from abc import ABC
from typing import Callable,List
from localstack import config
from localstack.packages import InstallTarget,PackageInstaller,SystemNotSupportedException,package
from localstack.utils.platform import in_docker,is_debian,is_redhat
from localstack.utils.run import run
LOG=logging.getLogger(__name__)
OS_PACKAGE_INSTALL_LOCK=threading.RLock()
_DEBIAN_CACHE_DIR=os.path.join(config.dirs.cache,'apt')
class OSPackageInstaller(PackageInstaller,ABC):
	def __init__(A,name,version):super().__init__(name,version,OS_PACKAGE_INSTALL_LOCK)
	def _get_install_dir(A,target):return A._os_switch(debian=A._debian_get_install_dir,redhat=A._redhat_get_install_dir,debian_fallback=True,target=target)
	@staticmethod
	def _os_switch(debian,redhat,debian_fallback=False,**A):
		if in_docker()and is_redhat():return redhat(**A)
		if is_debian()or not in_docker()and debian_fallback:return debian(**A)
		elif not in_docker():raise SystemNotSupportedException('OS level packages are only installed within docker containers.')
		else:raise SystemNotSupportedException('The current operating system is currently not supported.')
	def _prepare_installation(A,target):
		B=target
		if B!=InstallTarget.STATIC_LIBS:LOG.warning('%s will be installed as an OS package, even though install target is _not_ set to be static.',A.name)
		A._os_switch(debian=A._debian_prepare_install,redhat=A._redhat_prepare_install,target=B)
	def _install(A,target):A._os_switch(debian=A._debian_install,redhat=A._redhat_install,target=target)
	def _post_process(A,target):A._os_switch(debian=A._debian_post_process,redhat=A._redhat_post_process,target=target)
	def _get_install_marker_path(A,install_dir):return A._os_switch(debian=A._debian_get_install_marker_path,redhat=A._redhat_get_install_marker_path,debian_fallback=True,install_dir=install_dir)
	def _debian_get_install_dir(A,target):raise SystemNotSupportedException(f"There is no supported installation method for {A.name} on Debian.")
	def _debian_get_install_marker_path(A,install_dir):raise SystemNotSupportedException(f"There is no supported installation method for {A.name} on Debian.")
	def _debian_packages(A):raise SystemNotSupportedException(f"There is no supported installation method for {A.name} on Debian.")
	def _debian_prepare_install(A,target):run(A._debian_cmd_prefix()+['update'])
	def _debian_install(A,target):B=A._debian_packages();LOG.debug('Downloading packages %s to folder: %s',B,_DEBIAN_CACHE_DIR);C=A._debian_cmd_prefix()+['-d',_A]+B;run(C);C=A._debian_cmd_prefix()+[_A]+B;run(C)
	def _debian_post_process(A,target):0
	def _debian_cmd_prefix(A):return['apt',f"-o=dir::cache={_DEBIAN_CACHE_DIR}",f"-o=dir::cache::archives={_DEBIAN_CACHE_DIR}",'-y']
	def _redhat_get_install_dir(A,target):raise SystemNotSupportedException(f"There is no supported installation method for {A.name} on RedHat.")
	def _redhat_get_install_marker_path(A,install_dir):raise SystemNotSupportedException(f"There is no supported installation method for {A.name} on Redhat.")
	def _redhat_packages(A):raise SystemNotSupportedException(f"There is no supported installation method for {A.name} on RedHat.")
	def _redhat_prepare_install(A,target):0
	def _redhat_install(A,target):run(['dnf',_A,'-y']+A._redhat_packages())
	def _redhat_post_process(A,target):run(['dnf','clean','all'])
pro_package=functools.partial(package,scope='ext')


import abc,logging,os
from typing import Optional
import yaml
from localstack.utils.files import load_file,save_file
from localstack_ext.bootstrap.pods.constants import VERSION_FILE
from localstack_ext.bootstrap.pods.models import PodNode,Version
LOG=logging.getLogger(__name__)
class PodsSerializer(abc.ABC):
	@abc.abstractmethod
	def store_version(self,pod_object,path):0
	@abc.abstractmethod
	def retrieve_version(self,key,remote_path,local_path):0
class PodsSerializerYaml(PodsSerializer):
	def store_version(C,version,path):A=version;B=os.path.join(path,VERSION_FILE.format(version_no=A.version_number));save_file(B,A.to_yaml());return A.hash_ref
	def retrieve_version(E,key,remote_path,local_path):
		A=os.path.join(remote_path or local_path,key or'')
		if not os.path.isfile(A):LOG.debug('No revision file found in path %s',A);return
		D=load_file(A);B=yaml.safe_load(D);C=B.get('obj_type')
		if C==Version.__name__:return Version.from_dict(B)
		raise Exception(f"Unsupported object type '{C}'")


from __future__ import annotations
import logging
from typing import List,Optional
from localstack.constants import APPLICATION_OCTET_STREAM
from localstack.http import Response
from localstack_ext.bootstrap.pods.server import persistence
from localstack_ext.bootstrap.pods.server.states import ServiceStateManager
from localstack_ext.bootstrap.pods.service_state.service_state import ServiceState
from localstack_ext.bootstrap.pods.utils.adapters import ServiceStateMarshaller
LOG=logging.getLogger(__name__)
def handle_get_state_request_in_memory(services=None):
	from localstack.services.plugins import SERVICE_PLUGINS as C;D=C.list_loaded_services();B=ServiceState()
	for A in services or D:
		E=C.get_service_container(A)
		if not E:LOG.debug("Can't get service container for service %s while calling handle_get_state_request_in_memory",A)
		try:F=ServiceStateManager(service=A);G=F.get_service_state();B.put_service_state(G)
		except Exception as H:LOG.debug('Unable to retrieve the state for service %s: %s - skipping',A,H)
	if B.is_empty():LOG.debug('Extracted state is empty')
	I=ServiceStateMarshaller.marshall(state=B,marshall_function=persistence.marshall_backend);J=Response(I,mimetype=APPLICATION_OCTET_STREAM);return J


_B='spark-submit'
_A='2.2.1'
import glob,logging,os
from typing import Dict,List
from localstack import config
from localstack.constants import MAVEN_REPO_URL
from localstack.packages import InstallTarget,Package,PackageInstaller
from localstack.packages.core import ArchiveDownloadAndExtractInstaller
from localstack.utils.archives import unzip
from localstack.utils.files import cp_r,file_exists_not_empty,load_file,mkdir,new_tmp_dir,new_tmp_file,rm_rf,save_file
from localstack.utils.http import download
from localstack.utils.run import run
from localstack.utils.testutil import create_zip_file
from localstack_ext import config as ext_config
from localstack_ext.packages.java import java_package
LOG=logging.getLogger(__name__)
SPARK_URL='https://archive.apache.org/dist/spark/spark-{version}/spark-{version}-bin-without-hadoop.tgz'
DEFAULT_SPARK_VERSION='2.4.3'
SPARK_VERSIONS=[DEFAULT_SPARK_VERSION,_A,'2.4.8','3.1.1','3.1.2']
AWS_SDK_VER='1.12.339'
HADOOP_VERSION='2.9.2'
JAR_URLS=[f"{MAVEN_REPO_URL}/com/amazonaws/aws-java-sdk/{AWS_SDK_VER}/aws-java-sdk-{AWS_SDK_VER}.jar",f"{MAVEN_REPO_URL}/com/amazonaws/aws-java-sdk-core/{AWS_SDK_VER}/aws-java-sdk-core-{AWS_SDK_VER}.jar",f"{MAVEN_REPO_URL}/com/amazonaws/aws-java-sdk-glue/{AWS_SDK_VER}/aws-java-sdk-glue-{AWS_SDK_VER}.jar",f"{MAVEN_REPO_URL}/com/amazonaws/aws-java-sdk-s3/{AWS_SDK_VER}/aws-java-sdk-s3-{AWS_SDK_VER}.jar",f"{MAVEN_REPO_URL}/com/amazonaws/aws-java-sdk-dynamodb/{AWS_SDK_VER}/aws-java-sdk-dynamodb-{AWS_SDK_VER}.jar",f"{MAVEN_REPO_URL}/com/amazonaws/aws-java-sdk-lakeformation/{AWS_SDK_VER}/aws-java-sdk-lakeformation-{AWS_SDK_VER}.jar",f"{MAVEN_REPO_URL}/com/amazonaws/aws-java-sdk-bundle/{AWS_SDK_VER}/aws-java-sdk-bundle-{AWS_SDK_VER}.jar",f"{MAVEN_REPO_URL}/org/apache/hadoop/hadoop-hdfs/{HADOOP_VERSION}/hadoop-hdfs-{HADOOP_VERSION}.jar",f"{MAVEN_REPO_URL}/org/apache/hadoop/hadoop-common/{HADOOP_VERSION}/hadoop-common-{HADOOP_VERSION}.jar",f"{MAVEN_REPO_URL}/org/apache/hadoop/hadoop-auth/{HADOOP_VERSION}/hadoop-auth-{HADOOP_VERSION}.jar",f"{MAVEN_REPO_URL}/org/apache/hadoop/hadoop-aws/{HADOOP_VERSION}/hadoop-aws-{HADOOP_VERSION}.jar",f"{MAVEN_REPO_URL}/com/typesafe/config/1.3.3/config-1.3.3.jar",f"{MAVEN_REPO_URL}/com/fasterxml/jackson/dataformat/jackson-dataformat-csv/2.11.4/jackson-dataformat-csv-2.11.4.jar",f"{MAVEN_REPO_URL}/com/fasterxml/jackson/core/jackson-core/2.11.4/jackson-core-2.11.4.jar",f"{MAVEN_REPO_URL}/com/github/tony19/named-regexp/0.2.6/named-regexp-0.2.6.jar",f"{MAVEN_REPO_URL}/com/amazon/emr/emr-dynamodb-hadoop/4.12.0/emr-dynamodb-hadoop-4.12.0.jar",f"{MAVEN_REPO_URL}/de/undercouch/bson4jackson/2.11.0/bson4jackson-2.11.0.jar",f"{MAVEN_REPO_URL}/com/google/guava/guava/30.0-jre/guava-30.0-jre.jar",f"{MAVEN_REPO_URL}/org/apache/commons/commons-configuration2/2.7/commons-configuration2-2.7.jar",f"{MAVEN_REPO_URL}/commons-configuration/commons-configuration/1.10/commons-configuration-1.10.jar",f"{MAVEN_REPO_URL}/org/apache/commons/commons-text/1.9/commons-text-1.9.jar",f"{MAVEN_REPO_URL}/commons-lang/commons-lang/2.6/commons-lang-2.6.jar",f"{MAVEN_REPO_URL}/org/apache/logging/log4j/log4j-api/2.14.0/log4j-api-2.14.0.jar",f"{MAVEN_REPO_URL}/org/postgresql/postgresql/42.3.1/postgresql-42.3.1.jar",f"{MAVEN_REPO_URL}/com/google/guava/failureaccess/1.0.1/failureaccess-1.0.1.jar",f"{MAVEN_REPO_URL}/org/apache/hadoop/thirdparty/hadoop-shaded-protobuf_3_7/1.0.0/hadoop-shaded-protobuf_3_7-1.0.0.jar",f"{MAVEN_REPO_URL}/com/google/re2j/re2j/1.5/re2j-1.5.jar"]
REPO_URL_GLUE_LIBS='git+https://github.com/awslabs/aws-glue-libs.git#egg=aws-glue-libs'
REPO_URL_GLUE_LIBS_0_9='git+https://github.com/localstack/aws-glue-libs.git@glue-0.9#egg=aws-glue-libs'
JAR_FOLDER='/usr/local/gluePyspark'
if ext_config.BIGDATA_MONO_CONTAINER:JAR_FOLDER=os.path.join(config.dirs.var_libs,'gluePyspark')
class SparkPackage(Package):
	def __init__(A):super().__init__('Spark',default_version=DEFAULT_SPARK_VERSION)
	def get_versions(A):return SPARK_VERSIONS
	def _get_installer(A,version):return SparkInstaller(version)
class SparkInstaller(ArchiveDownloadAndExtractInstaller):
	def __init__(A,version):super().__init__(name='spark',version=version,extract_single_directory=True)
	def get_installed_dir(B):
		A=super().get_installed_dir()
		if A:
			if ext_config.BIGDATA_MONO_CONTAINER:return A
			from localstack_ext.utils import hadoop as C;D=get_spark_home_in_container(B.version)
			if C.path_exists_in_container(D):return A
	def _get_download_url(A):return SPARK_URL.format(version=A.version)
	def _get_install_marker_path(A,install_dir):return os.path.join(install_dir,'bin',_B)
	def _post_process(D,target):
		from localstack_ext.utils import hadoop as B;G=D._get_install_dir(target);K=D._get_install_marker_path(G);A=get_spark_home_in_container(D.version);P=os.path.join(A,'bin',_B)
		if not ext_config.BIGDATA_MONO_CONTAINER and B.path_exists_in_container(P):return
		if not os.path.exists(K):
			LOG.warning("Unable to find 'spark-submit' in the downloaded archive: %s",K)
			if not os.path.exists(G):LOG.warning('Target installation dir does not exist: %s',G)
		if not ext_config.BIGDATA_MONO_CONTAINER:LOG.debug('Copying Spark installation into bigdata container: %s',A);B.copy_into_spark_container(G,A)
		H=[]
		if not ext_config.BIGDATA_MONO_CONTAINER:H.append(['cp',f"{B.DEFAULT_SPARK_HOME}/conf/spark-env.sh",f"{A}/conf/spark-env.sh"])
		H.append(['sed','-ie','$s/^exec "\\${CMD\\[@]}"/if [ -n "$SPARK_OVERWRITE_CP" ]; then CMD[2]="$SPARK_OVERWRITE_CP:${CMD[2]}"; fi\\nCMD=("${CMD[0]}" "-Dcom.amazonaws.sdk.disableCertChecking=true" "${CMD[@]:1}"); exec "${CMD[@]}"/',f"{A}/bin/spark-class"])
		for I in H:B.run_in_presto(I)
		C=new_tmp_file();save_file(C,B.SPARK_DEFAULTS_CONF);B.copy_into_spark_container(C,f"{A}/conf/spark-defaults.conf");rm_rf(C)
		if ext_config.BIGDATA_MONO_CONTAINER:from localstack_ext.packages.hadoop import HadoopInstaller as Q,get_hadoop_home_in_container as R;L=get_hadoop_version_for_spark_version(D.version);Q(L).install();J=new_tmp_file();M=R(L);S=f'\n            export SPARK_DIST_CLASSPATH="$({M}/bin/hadoop classpath)"\n            export HADOOP_CONF_DIR="{M}/etc/hadoop"\n            ';save_file(J,S);B.copy_into_spark_container(J,f"{A}/conf/spark-env.sh");rm_rf(J)
		if ext_config.BIGDATA_MONO_CONTAINER:
			N=REPO_URL_GLUE_LIBS
			if D.version==_A:N=REPO_URL_GLUE_LIBS_0_9
			I=['pip','install',N];run(I)
		if ext_config.BIGDATA_MONO_CONTAINER:
			download_additional_jar_files();O=f"{A}/conf/spark-defaults.conf";C=load_file(O)
			if'/gluePyspark'not in C:C+=f"\nspark.driver.extraClassPath {JAR_FOLDER}/*\nspark.executor.extraClassPath {JAR_FOLDER}/*\nspark.driver.allowMultipleContexts = true\n";save_file(O,C)
		if ext_config.BIGDATA_MONO_CONTAINER:E=os.path.join(A,'python/lib/py4j-*-src.zip');E=glob.glob(E)[0];F={'from collections import':'from collections.abc import'};replace_in_zip_file(E,'py4j/java_collections.py',F);E=os.path.join(A,'python/lib/pyspark.zip');F={'import collections\n':'import collections.abc\n','collections.Iterable':'collections.abc.Iterable'};replace_in_zip_file(E,'pyspark/resultiterable.py',F);F={'_cell_set_template_code =':'# _cell_set_template_code ='};replace_in_zip_file(E,'pyspark/cloudpickle.py',F)
		if ext_config.BIGDATA_MONO_CONTAINER and D.version.startswith('2.'):java_package._get_installer('8').install()
	def copy_into_container(A):return A._post_process(InstallTarget.VAR_LIBS)
spark_package=SparkPackage()
def replace_in_zip_file(zip_file,file_path,search_replace,raise_if_missing=False):
	C=zip_file;A=new_tmp_dir();unzip(C,A);B=os.path.join(A,file_path)
	if not os.path.exists(B):
		if raise_if_missing:raise Exception(f"Unable to replace content in non-existing file in archive: {B}")
		return
	replace_in_file(B,search_replace);create_zip_file(A,C);rm_rf(A)
def replace_in_file(file_path,search_replace):
	B=file_path;A=load_file(B)
	for (C,D) in search_replace.items():A=A.replace(C,D)
	save_file(B,A)
def get_hadoop_version_for_spark_version(spark_version):
	from localstack_ext.packages.hadoop import HADOOP_DEFAULT_VERSION as A
	if ext_config.BIGDATA_MONO_CONTAINER and spark_version==_A:return'2.10.2'
	return A
def get_spark_install_cache_dir(spark_version):return spark_package.get_installer(spark_version)._get_install_dir(InstallTarget.VAR_LIBS)
def get_spark_home_in_container(spark_version):
	A=spark_version
	if ext_config.BIGDATA_MONO_CONTAINER:return get_spark_install_cache_dir(A)
	return f"/usr/local/spark-{A}"
def download_additional_jar_files():
	for A in JAR_URLS:download_and_cache_jar_file(A)
	return JAR_FOLDER
def download_and_cache_jar_file(jar_url):
	B=jar_url;mkdir(JAR_FOLDER);C=os.path.join(JAR_FOLDER,B.split('/')[-1])
	if file_exists_not_empty(C):return C
	A=os.path.join(config.dirs.var_libs,'bigdata-jars',B.split('/')[-1])
	if not file_exists_not_empty(A):download(B,A)
	cp_r(A,C);return A


import logging,os,traceback
from contextlib import contextmanager
from localstack import constants
from localstack.constants import ARTIFACTS_REPO
from localstack.packages import InstallTarget,PackageException
from localstack.utils.http import download
from localstack.utils.time import now
from localstack_ext.bootstrap.licensing import is_enterprise
LOG=logging.getLogger(__name__)
SSL_CERT_URL=f"{ARTIFACTS_REPO}/raw/master/local-certs/server.key"
SSL_CERT_URL_FALLBACK='{api_endpoint}/proxy/localstack.cert.key'
@contextmanager
def log_package_exception():
	try:yield
	except PackageException:LOG.exception('Package installation failed.')
def install_libs():
	from localstack_ext.packages.postgres import postgresql_package as B;from localstack_ext.services.elasticache.packages import redis_package as C;from localstack_ext.services.iot.packages import iot_rule_engine_package as D,mosquitto_package as E;from localstack_ext.services.stepfunctions.packages import stepfunctions_pro_package as F;from localstack_ext.services.timestream.packages import timescaledb_package as G;A=InstallTarget.STATIC_LIBS
	with log_package_exception():D.install(target=A)
	with log_package_exception():B.install(target=A)
	with log_package_exception():G.install(target=A)
	with log_package_exception():C.install(target=A)
	with log_package_exception():E.install(target=A)
	with log_package_exception():F.install(target=A)
def setup_ssl_cert():
	from localstack.services import generic_proxy as C;A=C.get_cert_pem_file_path()
	if os.path.exists(A):
		if is_enterprise():LOG.debug('Avoiding to update SSL certificate.');return
		D=6*60*60;E=os.path.getmtime(A)
		if E>now()-D:LOG.debug('Using cached SSL certificate (less than 6hrs since last update).');return
	LOG.debug('Attempting to download local SSL certificate file');F=3;G=5
	try:return download_github_artifact(SSL_CERT_URL,A,timeout=F)
	except Exception:
		B=SSL_CERT_URL_FALLBACK.format(api_endpoint=constants.API_ENDPOINT)
		try:return download(B,A,timeout=G)
		except Exception as H:LOG.info('Unable to download local test SSL certificate from %s to %s (using self-signed cert as fallback): %s',B,A,H);raise
def download_github_artifact(url,target_file,timeout=None):
	B=target_file;A=url
	def C(url,print_error=False):
		try:download(url,B,timeout=timeout);return True
		except Exception as A:
			if print_error:LOG.info('Unable to download Github artifact from from %s to %s: %s %s'%(url,B,A,traceback.format_exc()))
	D=C(A)
	if not D:A=A.replace('https://github.com','https://cdn.jsdelivr.net/gh');A=A.replace('/raw/master/','@master/');C(A,True)


_B=False
_A=None
import json,logging,os,threading,zipfile
from typing import List,Optional,Union
from localstack import config as localstack_config
from localstack.utils.files import load_file
from localstack.utils.http import safe_requests
from localstack_ext.bootstrap.auth import get_auth_cache
from localstack_ext.bootstrap.licensing import get_auth_headers
from localstack_ext.bootstrap.pods.constants import ASSETS_ROOT_DIR,COMMIT_FILE,COMPRESSION_FORMAT,DEFAULT_POD_DIR,HEAD_FILE,KNOWN_VER_FILE,MAX_VER_FILE,META_ZIP,OBJ_STORE_DIR,STATE_ZIP,VER_LOG_FILE,VER_LOG_STRUCTURE,VERSION_SPACE_DIRS,VERSION_SPACE_FILES,VERSIONS_DIR
LOCK_TYPE=type(threading.Lock())
RLOCK_TYPE=type(threading.RLock())
LOG=logging.getLogger(__name__)
class PodsConfigContext:
	def __init__(A,pods_root_dir=_A,pod_name=_A):A.cloud_pods_root_dir=pods_root_dir or A.get_pods_root_dir();A.pod_name=pod_name;A.user=_A
	@property
	def pod_root_dir(self):
		A=self
		if not A.pod_name:raise Exception('Unable to determine pod root dir as pod name is not configured')
		return os.path.join(A.cloud_pods_root_dir,A.pod_name)
	@classmethod
	def get_pod_config_dir(A,pod_name):return os.path.join(A.get_pods_root_dir(),pod_name)
	@classmethod
	def get_pods_root_dir(A):return os.environ.get('POD_DIR')or os.path.join(localstack_config.CONFIG_DIR,DEFAULT_POD_DIR)
	def get_pod_context(A):return os.path.basename(A.pod_root_dir)
	def get_context_user(A):return A.user
	def get_pod_root_dir(A):return A.pod_root_dir
	def get_head_path(A):return os.path.join(A.pod_root_dir,HEAD_FILE)
	def get_max_ver_path(A):return os.path.join(A.pod_root_dir,MAX_VER_FILE)
	def get_known_ver_path(A):return os.path.join(A.pod_root_dir,KNOWN_VER_FILE)
	def get_ver_log_path(A):return os.path.join(A.pod_root_dir,VER_LOG_FILE)
	def get_obj_store_path(A):return os.path.join(A.pod_root_dir,OBJ_STORE_DIR)
	def get_versions_path(A):return os.path.join(A.pod_root_dir,VERSIONS_DIR)
	def get_assets_root_path(A):return os.path.join(A.pod_root_dir,ASSETS_ROOT_DIR)
	def get_version_meta_archive_path(B,version,with_format=True):
		A=os.path.join(B.get_pod_root_dir(),META_ZIP.format(version_no=version))
		if not with_format:return A
		return f"{A}.{COMPRESSION_FORMAT}"
	def get_version_state_archive_path(B,version,with_format=True):
		A=os.path.join(B.get_pod_root_dir(),STATE_ZIP.format(version_no=str(version)))
		if not with_format:return A
		return f"{A}.{COMPRESSION_FORMAT}"
	def update_ver_log(A,author,ver_no,rev_id,rev_no):
		with open(A.get_ver_log_path(),'a')as B:B.write(f"{VER_LOG_STRUCTURE.format(author=author,ver_no=ver_no,rev_rid_no=f'{rev_id}_{rev_no}')}\n")
	def is_initialized(A):return A.pod_root_dir and os.path.isdir(A.pod_root_dir)
	def get_head_version_number(A):return int(A._read_file(A.get_head_path()))
	def get_max_version_number(A):return int(A._read_file(A.get_max_ver_path()))
	def list_known_versions(A):B=A._read_file(A.get_known_ver_path()).split('\n');C=[int(A)for A in B if A.strip()];return C
	def _read_file(A,path):return load_file(path).strip()
	def get_obj_file_path(A,key):return os.path.join(A.get_obj_store_path(),key)
	def is_remotely_managed(C,pod_name=_A):
		from localstack_ext.bootstrap.pods.remote import CloudPodsRemotePlatform as D;A=pod_name or C.pod_name
		try:E=D.create_platform_url();F=get_auth_headers();B=safe_requests.get(E,headers=F)
		except Exception as G:LOG.debug('An error occurred while checking remote management for pod "%s": "%s"',A,G);return _B
		if not B.ok:return _B
		H=json.loads(B.content);return A in[A['pod_name']for A in H]
	def set_pod_context(A,pod_name):
		B=pod_name;C=get_auth_cache();D=C.get('username','unknown');A.pod_name=B
		if not os.path.isdir(A.pod_root_dir):raise Exception(f"Unable to find local cloud pod named '{B}'")
		A.user=D
	def pod_exists_locally(A,pod_name):return os.path.isdir(os.path.join(A.cloud_pods_root_dir,pod_name))
	def rename_pod(A,new_pod_name):C=A.get_pod_root_dir();B=os.path.join(A.cloud_pods_root_dir,new_pod_name);os.rename(C,B);A.set_pod_context(B)
	def get_pod_name(A):return os.path.basename(A.get_pod_root_dir())
	def get_version_space_dir_paths(A):return[os.path.join(A.get_pod_root_dir(),B)for B in VERSION_SPACE_DIRS]
	def get_version_space_file_paths(A):return[os.path.join(A.get_pod_root_dir(),B)for B in VERSION_SPACE_FILES]
	def metamodel_file(B,revision,version=_A,absolute=_B):
		C=version;A=B.commit_metamodel_file(revision)
		if absolute:
			if C is _A:raise Exception('Missing pod version when constructing revision metamodel file path')
			A=os.path.join(B.metadata_dir(C),A)
		return A
	@staticmethod
	def commit_metamodel_file(commit_no):return COMMIT_FILE.format(commit_no=commit_no)
	def metadata_dir(A,version):return os.path.join(A.get_pod_root_dir(),META_ZIP.format(version_no=version))
	def get_version_meta_archive(B,version):
		A=B.get_version_meta_archive_path(version)
		if os.path.isfile(A):return A
	def get_version_state_archive(B,version):
		A=B.get_version_state_archive_path(version)
		if os.path.isfile(A):return A
def zip_directories(zip_dest,directories):
	A=zip_dest;from localstack.utils.archives import create_zip_file_python as C
	for B in directories:C(content_root=os.path.basename(B),base_dir=B,zip_file=A,mode='a')
	return A
def add_file_to_archive(archive,entry_name,content):
	with zipfile.ZipFile(archive,'a')as A:A.writestr(entry_name,content)
def read_file_from_archive(archive_path,file_name):
	B=file_name;A=archive_path
	try:
		with zipfile.ZipFile(A)as C:D=json.loads(C.read(B));return json.dumps(D)
	except Exception as E:LOG.debug(f"Could not find {B} in archive {A}: {E}")
def is_comma_delimited_list(string):
	import re;A=re.compile('^(\\w+)(,\\s*\\w+)*$')
	if A.match(string)is _A:return _B
	return True